#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Constraint', '_ConstraintData', 'ConstraintList', 'simple_constraint_rule', 'simple_constraintlist_rule']

import sys
import logging
import weakref
import StringIO

import pyutilib.math
import pyutilib.misc
from pyutilib.component.core import alias

from expr import Expression, generate_relational_expression, \
     _EqualityExpression, generate_expression_bypassCloneCheck
from numvalue import NumericValue, NumericConstant, ZeroConstant, value, \
     as_numeric, _sub
from indexed_component import IndexedComponent
from misc import create_name, apply_indexed_rule
from sets import Set
from objective import Objective
from component import Component

logger = logging.getLogger('coopr.pyomo')

_simple_constraint_rule_types = set([ type(None), bool ])

def simple_constraint_rule( fn ):
    """This is a decorator that translates None/True/False return values
    into Constraint.Skip/Constraint.Feasible/Constraint.Infeasible.
    This supports a simpler syntax in constraint rules, though these can be
    more difficult to debug when errors occur.

    Example use:

    @simple_constraint_rule
    def C_rule(model, i, j):
        ...
    """
    def wrapper_function ( *args, **kwargs ):
        if fn.__class__ in _simple_constraint_rule_types:
            value = fn
        else:
            value = fn( *args, **kwargs )
        if value.__class__ in _simple_constraint_rule_types:
            if value is None:
                return Constraint.Skip
            elif value is True:
                return Constraint.Feasible
            elif value is False:
                return Constraint.Infeasible
        return value
    return wrapper_function


def simple_constraintlist_rule( fn ):
    """This is a decorator that translates None/True/False return values
    into ConstraintList.End/Constraint.Feasible/Constraint.Infeasible.
    This supports a simpler syntax in constraint rules, though these can be
    more difficult to debug when errors occur.

    Example use:

    @simple_constraintlist_rule
    def C_rule(model, i, j):
        ...
    """
    def wrapper_function ( *args, **kwargs ):
        if fn.__class__ in _simple_constraint_rule_types:
            value = fn
        else:
            value = fn( *args, **kwargs )
        if value.__class__ in _simple_constraint_rule_types:
            if value is None:
                return ConstraintList.End
            elif value is True:
                return Constraint.Feasible
            elif value is False:
                return Constraint.Infeasible
        return value
    return wrapper_function



class _ConstraintData(NumericValue):
    """
    This class defines the data for a single constraint.

    Constructor arguments:
        name            The name of this constraint.  Use None for a default value.
        component       The Constraint object that owns this data.

    Public class attributes:
        active          A boolean that is true if this constraint is active in the model.
        component       The constraint component.
        domain          A domain object that restricts possible values for this constraint.
        expr            The Pyomo expression for this constraint
        id              An integer value that defines 
        index           The index value for this constraint
        name            The name of this constraint
        value           The numeric value of the constraint body

    Private class attributes:
        _repn           TBD
    """

    __slots__ = ('active', 'ampl_repn', 'body', 'component', 'dual', 
                 'index', 'lin_body', 'lower', 'name', 'repn', 'slack', 'upper', 'value',
                 '_equality')

    def __init__(self, name, component):
        # The following three lines are equivalent to calling the
        # basic NumericValue constructor, i.e., as follows:
        # NumericValue.__init__(self, name, None, None)
        self.name = name
        self.domain = None
        self.value = None
        #
        if component is None:
            self.component = None
        else:
            self.component = weakref.ref(component)
        self.active = True
        self.ampl_repn = None
        self.body = None 
        # an expression tree encoding of the constraint body. if None, an alternative representation (e.g., lin_body) will be != None.
        self.dual = None
        self.index = None
        self.lin_body = None # a linear compilation of the source expressions tree.
        self.lower = None
        self.repn = None
        self.slack = None
        self.upper = None
        self._equality = False

    def __getstate__(self):
        """
        This method must be defined because this class uses slots.
        """
        result = NumericValue.__getstate__(self)
        for i in _ConstraintData.__slots__:
            result[i] = getattr(self, i)
        if type(result['component']) is weakref.ref:
            result['component'] = result['component']()
        return result

    def __setstate__(self, state):
        """
        This method must be defined because this class uses slots.
        """
        result = NumericValue.__getstate__(self)
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)
        if self.component is not None:
            self.component = weakref.ref(self.component)

    def __call__(self, exception=True):
        """
        Compute the value of the body of this constraint.

        This method does not simply return self.value because that data value may be out of date
        w.r.t. the value of decision variables.
        """

        if self.body is None:
            return None
        return self.body()

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False


class Constraint(IndexedComponent, NumericValue):
    """
    An object that defines a objective expression

    Construct an objective expression with rule to construct the
    expression

    keyword arguments:
    name: name of this object
    rule: function or rule definition of constraint
    expr: same as rule
    doc:  documentation string for constraint
    """

    alias("Constraint", doc="Constraint expressions in a model.", subclass=True)

    NoConstraint    = (1000,)
    Skip            = (1000,)
    Infeasible      = (1001,)
    Violated        = (1001,)
    Feasible        = (1002,)
    Satisfied       = (1002,)

    def __new__(cls, *args, **kwds):
        if cls != Constraint:
            return super(Constraint, cls).__new__(cls)
        if args == ():
            return _ConstraintElement.__new__(_ConstraintElement)
        else:
            return _ConstraintArray.__new__(_ConstraintArray)

    def __init__(self, *args, **kwargs):
        tmprule = kwargs.pop('rule', None )
        tmprule = kwargs.pop('expr', tmprule )
        self.rule = tmprule
        self.trivial = False
        self._no_rule_init = kwargs.pop('noruleinit', None )
        self.domain = None
        self.value = None
        #
        kwargs.setdefault('ctype', Constraint)
        IndexedComponent.__init__(self, *args, **kwargs)

    def __str__(self):
        """
        Return a string representation of the constraint.  If the name attribute is None,
        then return ''.
        """
        if self.name is None:
            return ""
        else:
            return self.name

    def __getstate__(self):
        """
        This method is required because this class uses slots.  Call base class __getstate__
        methods since this class does not define additional slots.
        """
        numvalue_result = NumericValue.__getstate__(self)
        constraint_base_result = IndexedComponent.__getstate__(self)
        return dict(numvalue_result.items() + constraint_base_result.items())

    def __setstate__(self, state):
        """
        This method is required because this class uses slots.  Call base class __setstate__
        methods since this class does not define additional slots.
        """
        NumericValue.__setstate__(self, state)
        IndexedComponent.__setstate__(self, state)

    def construct(self, data=None):
        """TODO"""
        #
        # cache the debug generation flag, to avoid the expense of
        # calling isEnabledFor() for each index - this is far too
        # expensive an operation to perform deep in a loop. the only
        # potential down-side is that the debug logging is either
        # disabled or enabled globally for the duration of this
        # method invocation - which doesn't seem like much of
        # (if any) limitation in practice.
        #
        generate_debug_messages = (__debug__ is True) and (logger.isEnabledFor(logging.DEBUG) is True)

        if generate_debug_messages is True:
            logger.debug("Constructing constraint %s",self.name)

        if (self._no_rule_init is not None) and (self.rule is not None):
            logger.warn("noruleinit keyword is being used in conjunction with "
                        "rule keyword for constraint '%s'; defaulting to "
                        "rule-based construction.", self.name)
        if self.rule is None:
            if self._no_rule_init is None:
                logger.warn("No construction rule or expression specified for "
                            "constraint '%s'", self.name)
            return
        if self._constructed:
            return
        self._constructed=True
        #
        # Local variables for code optimization
        #
        _self_rule = self.rule
        _self_parent = self._parent()
        #
        # a somewhat gory way to see if you have a singleton - the len()=1 check is 
        # needed to avoid "None" being passed illegally into set membership validation rules.
        #
        if (len(self._index) == 1) and (None in self._index): 
            if len(self._index) != 1:
                raise IndexError, "Internal error: constructing constraint "\
                    "with both None and Index set"
            if generate_debug_messages:
                logger.debug("  Constructing single constraint (index=None)")
            if _self_rule.__class__ is bool or isinstance(_self_rule,Expression):
                expr = _self_rule
            else:
                expr = _self_rule(_self_parent)
            self.add(None, expr)
        else:
            if _self_rule.__class__ is bool or isinstance(_self_rule,Expression):
                raise IndexError, "Cannot define multiple indices in a " \
                    "constraint with a single expression"
            for index in self._index:
                if generate_debug_messages:
                    logger.debug("  Constructing constraint index "+str(index))
                self.add( index, apply_indexed_rule( self, _self_rule,
                                                     _self_parent, index ) )


    def add(self, index, expr):
        # index: the constraint index (should probably be renamed)
        # expr: the constraint expression

        expr_type = expr.__class__

        # Convert deprecated expression values
        #
        if expr_type in _simple_constraint_rule_types:
            if expr is None:
                raise ValueError("""
Invalid constraint expression.  The constraint expression resolved to
None instead of a Pyomo object.  Please modify your rule to return
Constraint.Skip instead of None.

Error thrown for constaint "%s"
""" % ( crete_name(self.name, index), ) )

            # There are cases where a user thinks they are generating a
            # valid 2-sided inequality, but Python's internal systems
            # for handling chained inequalities is doing something very
            # different and resolving it to True/False.  In this case,
            # chainedInequality will be non-None, but the expression
            # will be a bool.  For example, model.a < 1 > 0.
            if generate_relational_expression.chainedInequality is not None:
                buf = StringIO.StringIO()
                generate_relational_expression.chainedInequality.pprint(buf)
                # We are about to raise an exception, so it's OK to reset chainedInequality
                generate_relational_expression.chainedInequality = None
                raise ValueError("""
Invalid chained (2-sided) inequality detected.  The expression is
resolving to %s instead of a Pyomo Expression object.  This can occur
when the middle term of a chained inequality is a constant or immutable
parameter, for example, "model.a <= 1 >= 0".  The proper form for
2-sided inequalities is "0 <= model.a <= 1".

Error thrown for constaint "%s"

Unresolved (dangling) inequality expression:
    %s
""" % ( expr, crete_name(self.name, index), buf ) )
            else:
                raise ValueError("""
Invalid constraint expression.  The constraint expression resolved to a
trivial Boolean (%s) instead of a Pyomo object.  Please modify your rule
to return Constraint.%s instead of %s.

Error thrown for constaint "%s"
""" % ( expr, expr and "Feasible" or "Infeasible", expr, crete_name(self.name, index) ) )

        #
        # Ignore an 'empty' constraint
        #
        if expr_type is tuple and len(expr) == 1:
            if expr == Constraint.Skip or expr == Constraint.Feasible:
                return
            if expr == Constraint.Infeasible:
                raise ValueError, "Constraint '%s' is always infeasible" % create_name(self.name,index)
        #
        #
        # Local variables to optimize runtime performance
        #
        if index is None:
            conData = self
        else:
            conData = _ConstraintData(create_name(self.name,index), self)
        conData.index = index
        #
        if expr_type is tuple: # or expr_type is list:
            #
            # Form equality expression
            #
            if len(expr) == 2:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                conData._equality = True
                if arg1 is None or arg1.is_fixed():
                    conData.lower = conData.upper = arg1
                    conData.body = arg0
                elif arg0 is None or arg0.is_fixed():
                    conData.lower = conData.upper = arg0
                    conData.body = arg1
                else:
                    conData.lower = conData.upper = ZeroConstant
                    conData.body = arg0 - arg1
            #
            # Form inequality expression
            #
            elif len(expr) == 3:
                arg0 = expr[0]
                if arg0 is not None:
                    arg0 = as_numeric(arg0)
                    if not arg0.is_fixed():
                        msg = "Constraint '%s' found a 3-tuple (lower, " \
                              "expression, upper) but the lower value was "\
                              "non-constant"
                        raise ValueError, msg % (conData.name,)

                arg1 = expr[1]
                if arg1 is not None:
                    arg1 = as_numeric(arg1)

                arg2 = expr[2]
                if arg2 is not None:
                    arg2 = as_numeric(arg2)
                    if not arg2.is_fixed():
                        msg = "Constraint '%s' found a 3-tuple (lower, " \
                              "expression, upper) but the upper value was "\
                              "non-constant"
                        raise ValueError, msg % (conData.name,)

                conData.lower = arg0
                conData.body  = arg1
                conData.upper = arg2
            else:
                msg = "Constructor rule for constraint '%s' returned a tuple" \
                      ' of length %d.  Expecting a tuple of length 2 or 3:\n' \
                      'Equality:   (left, right)\n' \
                      'Inequality: (lower, expression, upper)'
                raise ValueError, msg % ( self.name, len(expr) )

            relational_expr = False
        else:
            try:
                relational_expr = expr.is_relational()
                if not relational_expr:
                    msg = "Constraint '%s' does not have a proper value.  " \
                          "Found '%s'\nExpecting a tuple or equation.  " \
                          "Examples:\n" \
                          "    summation( model.costs ) == model.income\n" \
                          "    (0, model.price[ item ], 50)"
                    raise ValueError, msg % ( conData.name, str(expr) )
            except AttributeError:
                msg = "Constraint '%s' does not have a proper value.  " \
                      "Found '%s'\nExpecting a tuple or equation.  " \
                      "Examples:\n" \
                      "    summation( model.costs ) == model.income\n" \
                      "    (0, model.price[ item ], 50)" \
                      % ( conData.name, str(expr) )
                if type(expr) is bool:
                    msg +="""
Note: constant Boolean expressions are not valid constraint expressions.
Some apparently non-constant compound inequalities (e.g. "expr >= 0 <= 1")
can return boolean values; the proper form for compound inequalities is
always "lb <= expr <= ub"."""
                raise ValueError, msg
        #
        # Special check for chainedInequality errors like "if var < 1:"
        # within rules.  Catching them here allows us to provide the
        # user with better (and more immediate) debugging information.
        # We don't want to check earlier because we want to provide a
        # specific debugging message if the construction rule returned
        # True/False; for example, if the user did ( var < 1 > 0 )
        # (which also results in a non-None chainedInequality value)
        #
        if generate_relational_expression.chainedInequality is not None:
            from expr import chainedInequalityErrorMessage
            raise TypeError, chainedInequalityErrorMessage()
        #
        # Process relational expressions (i.e. explicit '==', '<', and '<=')
        #
        if relational_expr:
            if expr_type is _EqualityExpression:
                # Equality expression: only 2 arguments!
                conData._equality = True
                if expr._args[1].is_fixed():
                    conData.lower = conData.upper = expr._args[1]
                    conData.body = expr._args[0]
                elif expr._args[0].is_fixed():
                    conData.lower = conData.upper = expr._args[0]
                    conData.body = expr._args[1]
                else:
                    conData.lower = conData.upper = ZeroConstant
                    conData.body = generate_expression_bypassCloneCheck(_sub, expr._args[0], expr._args[1])
            else:
                # Inequality expression: 2 or 3 arguments
                if len(expr._args) == 3:
                    if not expr._args[0].is_fixed():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the lower bound was non-constant"
                        raise ValueError, msg % (conData.name,)
                    if not expr._args[2].is_fixed():
                        msg = "Constraint '%s' found a double-sided "\
                              "inequality expression (lower <= expression "\
                              "<= upper) but the upper bound was non-constant"
                        raise ValueError, msg % (conData.name,)
                    conData.lower = expr._args[0]
                    conData.body  = expr._args[1]
                    conData.upper = expr._args[2]
                else:
                    if expr._args[1].is_fixed():
                        conData.lower = None
                        conData.body  = expr._args[0]
                        conData.upper = expr._args[1]
                    elif expr._args[0].is_fixed():
                        conData.lower = expr._args[0]
                        conData.body  = expr._args[1]
                        conData.upper = None
                    else:
                        conData.lower = None
                        conData.body  = generate_expression_bypassCloneCheck(_sub, expr._args[0], expr._args[1])
                        conData.upper = ZeroConstant
        #
        # Replace numeric bound values with a NumericConstant object,
        # and reset the values to 'None' if they are 'infinite'
        #
        if conData.lower is not None:
            val = conData.lower()
            if not pyutilib.math.is_finite(val):
                if val > 0:
                    msg = "Constraint '%s' created with a +Inf lower bound"
                    raise ValueError, msg % ( conData.name, )
                conData.lower = None
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric lower bound"
                raise ValueError, msg % ( conData.name, )
        if conData.upper is not None:
            val = conData.upper()
            if not pyutilib.math.is_finite(val):
                if val < 0:
                    msg = "Constraint '%s' created with a -Inf upper bound"
                    raise ValueError, msg % ( conData.name, )
                conData.upper = None
            elif bool(val > 0) == bool(val <= 0):
                msg = "Constraint '%s' created with a non-numeric upper bound"
                raise ValueError, msg % ( conData.name, )
        #
        # Error check, to ensure that we don't have a constraint that
        # doesn't depend on any variables / parameters
        #
        # Error check, to ensure that we don't have an equality constraint with
        # 'infinite' RHS
        #
        if conData._equality:
            if conData.lower != conData.upper: #pragma:nocover
                msg = "Equality constraint '%s' has non-equal lower and "\
                      "upper bounds (this is indicitive of a SERIOUS "\
                      "internal error in Pyomo)."
                raise RuntimeError, msg % self.name
            if conData.lower is None:
                msg = "Equality constraint '%s' defined with non-finite term"
                raise ValueError, msg % self.name
        #
        # hook up the constraint data object to the parent constraint.
        #
        self._data[index] = conData

    def pprint(self, ostream=None, verbose=False):
        """TODO"""
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        if not self.doc is None:
            print >>ostream, self.doc
            print >>ostream, "  ",
        print >>ostream, "\tSize="+str(len(self._data.keys())),
        if isinstance(self._index,Set):
            print >>ostream, "\tIndex=",self._index.name
        else:
            print >>ostream,""
        for val in self._data:
            if not val is None:
                print >>ostream, "\t"+`val`
            if self._data[val].lower is not None:
                print >>ostream, "\t\t",
                if self._data[val].lower.is_expression():
                    self._data[val].lower.pprint(ostream)
                else:
                    print >>ostream, str(self._data[val].lower)
            else:
                print >>ostream, "\t\t-Inf"
            print >>ostream, "\t\t<="
            if self._data[val].body is not None:
                print >>ostream, "\t\t",
                if self._data[val].body.is_expression():
                    self._data[val].body.pprint(ostream)
                else:
                    print >>ostream, str(self._data[val].body)
            print >>ostream, "\t\t<="
            if self._data[val].upper is not None:
                print >>ostream, "\t\t",
                if self._data[val].upper.is_expression():
                    self._data[val].upper.pprint(ostream)
                else:
                    print >>ostream, str(self._data[val].upper)
            elif self._data[val]._equality:
                print >>ostream, "\t\t",
                if self._data[val].lower.is_expression():
                    self._data[val].lower.pprint(ostream)
                else:
                    print >>ostream, str(self._data[val].lower)
            else:
                print >>ostream, "\t\tInf"

    def display(self, prefix="", ostream=None):
        """TODO"""
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, prefix+"Constraint "+self.name,":",
        print >>ostream, "  Size="+str(len(self))
        if None in self._data:
            if self._data[None].body is None:
                val = 'none'
            else:
                val = pyutilib.misc.format_io(self._data[None].body())
            print >>ostream, '%s  Value=%s' % (prefix, val)
        else:
            flag=True
            for key in self._data:
                if not self._data[key].active:
                    continue
                if flag:
                    print >>ostream, prefix+"        \tLower\tBody\t\tUpper"
                    flag=False
                if self._data[key].lower is not None:
                    lval = str(self._data[key].lower())
                else:
                    lval = "-Infinity"
                val = str(self._data[key].body())
                if self._data[key].upper is not None:
                    uval = str(self._data[key].upper())
                else:
                    uval = "Infinity"
                print >>ostream, "%s  %s :\t%s\t%s\t%s" % (
                                 prefix, str(key), lval, val, uval )
            if flag:
                print >>ostream, prefix+"  None active"


class _ConstraintElement(Constraint, _ConstraintData):

    def __init__(self, *args, **kwd):
        _ConstraintData.__init__(self, kwd.get('name', None), self)
        Constraint.__init__(self, *args, **kwd)

    def __call__(self, exception=True):
        """Compute the value of the constraint body"""
        if self.body is None:
            return None
        return value(self.body)

    def __getstate__(self):
        #
        # picks up all slots in the _ConstraintData class, and in particular
        # makes sure the "component" value is not a weakref,
        # but the actual objects themselves.
        #
        result1 = _ConstraintData.__getstate__(self)
        result1['component'] = self
        result2 = Constraint.__getstate__(self)
        return dict(result1.items() + result2.items())

    def __setstate__(self, state):
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)

        self.component = weakref.ref(self)
        # this is hackish, in that it replicates code from the base Component setstate method - 
        # revisit at   some point.
        if '_parent' in self.__dict__.keys() and self._parent is not None and type(self._parent) != weakref.ref:
            self._parent = weakref.ref(self._parent)


class _ConstraintArray(Constraint):

    def __call__(self, exception=True):
        """Compute the value of the constraint body"""
        if exception:
            msg = 'Cannot compute the value of an array of constraints'
            raise ValueError, msg


class ConstraintList(Constraint):
    """
    A constraint component that represents a list of constraints.  Constraints can
    be indexed by their index, but when they are added an index value is not specified.
    """

    alias("ConstraintList", "A list of constraints in a model.")

    End             = (1003,)

    def __init__(self, *args, **kwargs):
        """Constructor"""
        if len(args) > 0:
            raise ValueError, "Cannot specify indices for a ConstraintList object"
        #
        self._hidden_index = Set()
        targs = [self._hidden_index]
        self._nconstraints = 0
        kwargs.setdefault('ctype', ConstraintList)
        Constraint.__init__(self, *targs, **kwargs)

    def construct(self, *args, **kwds):
        """TODO"""
        self._hidden_index.construct()
        Constraint.construct(self, *args, **kwds)

    def construct(self, data=None):
        """TODO"""
        #
        generate_debug_messages = (__debug__ is True) and \
                                  (logger.isEnabledFor(logging.DEBUG) is True)

        if generate_debug_messages is True:
            logger.debug("Constructing constraint list %s",self.name)

        if (self._no_rule_init is not None) and (self.rule is not None):
            msg = 'WARNING: noruleinit keyword is being used in conjunction ' \
                  "with rule keyword for constraint '%s'; defaulting to "     \
                  'rule-based construction.'
            print msg % self.name
        if self.rule is None:
            return
        if self._constructed:
            return
        self._constructed=True
        #
        # Local variables for code optimization
        #
        _self_rule = self.rule
        _self_parent = self._parent()
        #
        while True:
            val = self._nconstraints + 1
            if generate_debug_messages:
                logger.debug("   Constructing constraint index "+str(val))
            expr = apply_indexed_rule( self, _self_rule, _self_parent, val )
            if expr is None:
                #logger.warning("DEPRECATION WARNING: Constraint rule returned None instead of ConstraintList.End")
                raise ValueError, "Constraint rule returned None instead of ConstraintList.End"
            if (expr.__class__ is tuple and expr == ConstraintList.End) or \
               (type(expr) in (int, long, float) and expr == 0):
                break
            self.add(expr)

    def add(self, *args):
        """TODO"""
        self._nconstraints += 1
        targs = [self._nconstraints] + list(args)
        Constraint.add(self, *targs)




from coopr.pyomo import *

model = AbstractModel()
model.S = RangeSet(10)
model.X = Var(model.S)

if sum(model.X[i] for i in model.S) <= 10.0:
    pass
if sum(value(model.X[i]) for i in model.S) <= 10.0:
    pass



from coopr.pyomo import *

model = ConcreteModel()
model.X = Var()

def c_rule(m):
    try:
        return model.X >= 10.0
    except Exception:
        return model.X >= 20.0
model.C = Constraint(rule=c_rule)



from pyutilib.component.core import *
from hooks import *

import re
import textwrap

class IModelChecker(Interface):
    def check(self, runner, script, info):
        """
        Check a particular piece of Python information for errors.

        Provides the primary interface for checking some code for problems,
        according to the particular subclass's definition. 
        
        @param runner the ModelCheckRunner instance that has dispatched
                      this call to check().
        @param script the ModelScript instance being checked.
        @param info the data to check. Depending on the subclass, info
                    can be the raw text of a Python script, the entire AST
                    of the script, or a particular node in that AST.
        """

    def beginChecking(self, runner, script):
        """
        Start checking the given script from the given runner.
        """

    def endChecking(self, runner, script):
        """
        Finish checking the given script from the given runner.
        """

    def problem(self, script = None, message = "Error", lineno = None):
        """
        Write a problem to the console. The format varies and can be
        changed in subclasses; by default, this method prints the following:

        [CheckerName] script.py:line: Error

        @param script the ModelScript instance being checked. Must be passed
                      to have the file name and line number printed.
        @param message the error to display.
        @param lineno the line number on which the error occurred.
        """

class PyomoModelChecker(SingletonPlugin):
    implements(IModelChecker, inherit=True)

    _prehooks = ExtensionPoint(IPreCheckHook)
    _posthooks = ExtensionPoint(IPostCheckHook)

    def __init__(self):
        self._currentRunner = None
        self._currentScript = None

    def _check(self, runner, script, info):
        self._runner = runner
        self._script = script
        
        for prehook in self._prehooks:
            prehook.precheck(runner, script, info)

        try:
            self.check(runner, script, info)
        except Exception as e:
            print self.checkerLabel() + "ERROR during check call!"
            raise e

        for posthook in self._posthooks:
            posthook.postcheck(runner, script, info)

        self._runner = None
        self._script = None

    def check(self, runner, script, info):
        # Should be `pass` - checkers are not guaranteed to call
        # superclass when running their own check() methods
        pass

    def _beginChecking(self, runner, script):
        self._currentRunner = runner
        self._currentScript = script

        try:
            self.beginChecking(runner, script)
        except Exception, e:
            print self.checkerLabel() + "ERROR during pre-check call!"

    def beginChecking(self, runner, script):
        pass

    def _endChecking(self, runner, script):
        try:
            self.endChecking(runner, script)
        except Exception, e:
            print self.checkerLabel() + "ERROR during pre-check call!"

        self._currentRunner = None
        self._currentScript = None

    def endChecking(self, runner, script):
        pass

    def _checkerName(self):
        match = re.search("<class '([a-zA-Z0-9_\.]+)'>", str(self.__class__))
        return match.group(1).split(".")[-1]

    def _checkerPackage(self):
        match = re.search("<class '([a-zA-Z0-9_\.]+)'>", str(self.__class__))
        return match.group(1).split(".")[-3]
        
    def checkerLabel(self):
        return "[" + self._checkerPackage() + "::" + self._checkerName() + "] "

    def checkerDoc(self):
        return ""

    def problem(self, message = "Error", runner = None, script = None, lineno = None):
        if script is None:
            script = self._currentScript
        if runner is None:
            runner = self._currentRunner

        output = self.checkerLabel()

        if script is not None:
            output += script.filename() + ":"
            if lineno is not None:
                output += str(lineno) + ":"
        else:
            output += "<unknown>:"

        output += " " + message

        print(output)

        try:
            if runner.verbose:
                if len(self.checkerDoc()) > 0:
                    lines = textwrap.dedent(self.checkerDoc()).split("\n")
                    lines = filter((lambda x : len(x) > 0), lines)
                    for line in lines:
                        print self.checkerLabel() + line
                print
        except Exception as e:
            print self.checkerLabel() + "ERROR during verbose info generation"
            print

class ImmediateDataChecker(PyomoModelChecker):
    pass

class IterativeDataChecker(PyomoModelChecker):
    pass

class ImmediateTreeChecker(PyomoModelChecker):
    pass

class IterativeTreeChecker(PyomoModelChecker):
    pass

from checkers import *



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['TableData']

from coopr.pyomo.base.plugin import IDataManager
from process_data import _process_data
from pyutilib.component.core import Plugin, implements
from pyutilib.misc import Options


class TableData(Plugin):
    """
    An object that imports data from a table in an external data source.
    """

    implements(IDataManager)

    def __init__(self):
        """
        Constructor
        """
        self._info=None
        self._data=None

    def initialize(self, filename, **kwds):
        self.filename = filename
        self.options = Options(**kwds)

    def open(self):
        """
        Open the table
        """
        pass

    def read(self):
        """
        Read data from the table
        """
        pass

    def close(self):
        """
        Close the table
        """
        pass

    def process(self, model, data, default):
        """
        Return the data that was extracted from this table
        """
        if not None in data:
            data[None] = {}
        return _process_data(
          self._info,
          model,
          data[None],
          default,
          self.filename
        )

    def clear(self):
        """
        Clear the data that was extracted from this table
        """
        self._info = None

    def _set_data(self, headers, rows):
        #print "_SET_DATA",headers,rows # XXX
        if self.options.param_name is None:
            mapped_headers = map(str,list(headers))
        else:
            mapped_headers = list()
            for header in map(str, list(headers)):
                if header in self.options.param_name:
                    mapped_headers.append(self.options.param_name[header])
        #print "X", mapped_headers, self.options.param_name, \
        #           self.options.index, self.options.format

        if self.options.format == 'set':
            if self.options.set is None:
                msg = "Must specify set name for data with the 'set' format"
                raise IOError, msg

            if not (self.options.index is None or self.options.index == []):
                msg = "Cannot specify index for data with the 'set' format: %s"
                raise IOError, msg % str( self.options.index )

            self._info = ["set",self.options.set,":="]
            for row in rows:
                self._info.extend(row)

        elif self.options.format == 'set_array':
            if self.options.set is None:
                msg = "Must specify set name for data with the 'set_array' "  \
                      'format'
                raise IOError, msg

            if not (self.options.index is None or self.options.index == []):
                msg = "Cannot specify index for data with the 'set_array' "   \
                      'format: %s'
                raise IOError, msg % str( self.options.index )

            self._info = ["set",self.options.set, ":"]
            self._info = self._info + list(headers[1:])
            self._info = self._info + [":="]
            for row in rows:
                self._info = self._info + list(row)

        elif self.options.format == 'transposed_array':
            self._info = ["param",self.options.param,"(tr)",":"]              \
                       + headers[1:]
            self._info.append(":=")
            for row in rows:
                self._info.extend(row)

        elif self.options.format == 'array':
            self._info = ["param",self.options.param,":"] + headers[1:]
            self._info.append(":=")
            for row in rows:
                self._info.extend(row)

        elif self.options.format == 'param':
            self._info = ["param",":",self.options.param,':=']
            for row in rows:
                self._info.extend(row)

        elif self.options.format == 'table' or self.options.format is None:
            #print "TABLE FORMAT" # XXX
            #print "INDEX:", self.options.index # XXX
            #print "INDEX NAME:", self.options.index_name # XXX
            #print "MAPPED HEADERS:", mapped_headers # XXX
            if self.options.index is None or len(self.options.index) == 0:
                msg = "Cannot import a relational table without specifying "  \
                      "index values"
                raise IOError, msg

            if self.options.index_name is not None:
                self._info = ["param",":",self.options.index_name,":"]
            else:
                self._info = ["param",":"]
            for header in mapped_headers:
                if not header in self.options.index:
                    self._info.append(header)
            self._info.append(":=")
            for row in rows:
                self._info.extend(row)
        else:
            msg = "Unknown parameter format: '%s'"
            raise ValueError, msg % self.options.format
        #print "FINAL",self._info # XXX



#
# Example 2.2 - Allen Holder
#

from coopr.pyomo import *

# Instantiate the model
model = AbstractModel()

# Parameters for Set Definitions
model.NumTimePeriods = Param(within=NonNegativeIntegers)

# Sets
model.StartTime = RangeSet(1,model.NumTimePeriods)

# Parameters
model.RequiredWorkers = Param(model.StartTime, within=NonNegativeIntegers)

# Variables
model.NumWorkers = Var(model.StartTime, within=NonNegativeIntegers)

# Objective
def CalcTotalWorkers(M):
    return sum (M.NumWorkers[i] for i in M.StartTime)
model.TotalWorkers = Objective(rule=CalcTotalWorkers, sense=minimize)

# Constraints
def EnsureWorkforce(M, i):
    if i != M.NumTimePeriods.value:
        return M.NumWorkers[i] + M.NumWorkers[i+1] >= M.RequiredWorkers[i+1]
    else:
        return M.NumWorkers[1] + M.NumWorkers[M.NumTimePeriods.value] \
               >= M.RequiredWorkers[1]
model.WorkforceDemand = Constraint(model.StartTime, rule=EnsureWorkforce)



from coopr.pyomo import *
from pyutilib.component.core import implements, Plugin, Interface
from coopr.pyomo.base import IModelTransformation

__all__ = ["Transformation"]

class Transformation(Plugin):
    """
    Base class for all model transformations.
    """

    implements(IModelTransformation)

    def __init__(self, **kwds):
        kwds["name"] = kwds.get("name", "transformation")
        super(Transformation, self).__init__(**kwds)

    def __call__(self, model, **kwds):
        """ Apply the transformation """

    def apply(self, model, **kwds):
        """ Alias for __call__ """



import ast

from checker import *
from script import ModelScript

class CheckingNodeVisitor(ast.NodeVisitor):
    
    def __init__(self, runner, script, tc = [], dc = [], pt = ""):
        """
        @param tc iterative tree checkers
        @param dc iterative data checkers
        @param pt program text
        """

        super(CheckingNodeVisitor, self).__init__()

        self.runner = runner
        self.script = script
        self.treeCheckers = tc
        self.dataCheckers = dc
        self.programLines = pt.split("\n")
        self.running_lineno = 0

    def sendBegin(self):
        for checker in self.treeCheckers + self.dataCheckers:
            checker._beginChecking(self.runner, self.script)

    def sendEnd(self):
        for checker in self.treeCheckers + self.dataCheckers:
            checker._endChecking(self.runner, self.script)

    def generic_visit(self, node):
        if 'lineno' in dir(node):
            current_lineno = node.lineno
            if current_lineno > self.running_lineno:
                self.running_lineno = current_lineno
                for checker in self.dataCheckers:
                    checker._check(self.runner, self.script, (current_lineno, self.programLines[current_lineno - 1]))

        for checker in self.treeCheckers:
            checker._check(self.runner, self.script, node)

        super(CheckingNodeVisitor, self).generic_visit(node)

class ModelCheckRunner:

    _checkers = ExtensionPoint(IModelChecker)

    def __init__(self):
        self.scripts = []

    def run(self, *args, **kwargs):
        # Get args
        script = kwargs.pop("script", None)
        verbose = kwargs.pop("verbose", False)
        checkers = kwargs.pop("checkers", {})
        show_checkers = kwargs.pop("show_checkers", False)

        # Store args as necessary
        self.verbose = verbose

        # Add script, if given
        if script is not None:
            self.addScript(ModelScript(script))

        # Enable listed checkers
        if checkers == {}:
            print("WARNING: No checkers enabled!")
        for c in self._checkers(all=True):
            if c._checkerPackage() in checkers:
                if c._checkerName() in checkers[c._checkerPackage()]:
                    c.enable()
                else:
                    c.disable()
            else:
                c.disable()

        # Show checkers if requested
        if show_checkers:
            printable = {}
            for c in self._checkers():
                if c._checkerPackage() not in printable:
                    printable[c._checkerPackage()] = [c._checkerName()]
                else:
                    printable[c._checkerPackage()].append(c._checkerName())
            
            for package in printable:
                print "{0}: {1}".format(package, " ".join(printable[package]))
            print

        # Pre-partition checkers
        immDataCheckers = [c for c in self._checkers if isinstance(c, ImmediateDataChecker)]
        iterDataCheckers = [c for c in self._checkers if isinstance(c, IterativeDataChecker)]
        immTreeCheckers = [c for c in self._checkers if isinstance(c, ImmediateTreeChecker)]
        iterTreeCheckers = [c for c in self._checkers if isinstance(c, IterativeTreeChecker)]

        for script in self.scripts:
            # Read in the script and call data checkers
            data = script.read()
            for checker in immDataCheckers:
                checker._beginChecking(self, script)
                checker._check(self, script, data)
                checker._endChecking(self, script)

            # Get the data into a parse tree
            tree = ast.parse(data)
            for checker in immTreeCheckers:
                checker._beginChecking(self, script)
                checker._check(self, script, tree)
                checker._endChecking(self, script)

            # Start walking the tree, calling checkers along the way
            visitor = CheckingNodeVisitor(self, script, tc=iterTreeCheckers, dc=iterDataCheckers, pt = data)
            visitor.sendBegin()
            visitor.visit(tree)
            visitor.sendEnd()

    def addScript(self, script):
        self.scripts.append(script)



#
# Example 2.5 - Allen Holder
#

from coopr.pyomo import *
from coopr.opt import *

# Instantiate the model
model = AbstractModel()

# Sets
model.NumMonths = Param(within=NonNegativeIntegers)
model.EngineType= Set()
model.Month = RangeSet(1,model.NumMonths)

# Parameters
model.Demand = Param(model.EngineType, model.Month, within=NonNegativeIntegers)
model.InvCost = Param(within=NonNegativeReals)
model.InitInv = Param(model.EngineType, within=NonNegativeIntegers)
model.FinInv = Param(model.EngineType, within=NonNegativeIntegers)
model.Labor = Param(model.EngineType, within=NonNegativeReals)
model.LaborBound = Param(within=NonNegativeReals)
model.ProdCost = Param(model.EngineType, within=NonNegativeReals)
model.ProdBound = Param(within=NonNegativeIntegers)

# Variables
model.Produce = Var(model.EngineType, model.Month, within=NonNegativeIntegers)
model.Inventory = Var(model.EngineType, model.Month, within=NonNegativeIntegers)

# Objective
def CalcCost(M):
    return sum(M.Produce[e,t]*M.ProdCost[e] \
               for e in M.EngineType for t in M.Month) + \
           sum(M.Inventory[e,t]*M.InvCost \
               for e in M.EngineType for t in M.Month)
model.TotalCost = Objective(rule=CalcCost, sense=minimize)

# Constraints
def EnsureBalance(M,e,t):
    if t != 1:
        return M.Inventory[e,t] == M.Inventory[e, t-1] + M.Produce[e,t] \
                                  - M.Demand[e,t]
    else:
        return M.Inventory[e,t] == M.InitInv[e] + M.Produce[e,t] \
                                  - M.Demand[e,t]
model.InventoryBalance = Constraint(model.EngineType, model.Month, rule=EnsureBalance)

def EnsureLaborLimit(M,t):
    return sum(M.Produce[e,t]*M.Labor[e] for e in M.EngineType) <= M.LaborBound
model.LimitLabor = Constraint(model.Month, rule=EnsureLaborLimit)

def EnsureProdLimit(M,t):
    return sum(M.Produce[e,t] for e in M.EngineType) <= M.ProdBound
model.ProdLimit = Constraint(model.Month, rule=EnsureProdLimit)

def LeaveEnough(M,e,t):
    if t == len(M.Month):
        return M.Inventory[e,t] >= M.FinInv[e]
model.FinalInventory = Constraint(model.EngineType, model.Month, rule=LeaveEnough)



#
# Test the Pyomo command-line interface
#

import unittest
import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep
scriptdir = dirname(dirname(dirname(dirname(dirname(abspath(__file__))))))+os.sep

import coopr.pyomo
import pyutilib.services
import pyutilib.subprocess
import pyutilib.th as unittest
import coopr.pyomo.scripting.convert as main
import StringIO
from pyutilib.misc import setup_redirect, reset_redirect
import re

if os.path.exists(sys.exec_prefix+os.sep+'bin'+os.sep+'coverage'):
    executable=sys.exec_prefix+os.sep+'bin'+os.sep+'coverage -x '
else:
    executable=sys.executable

def filter_fn(line):
    return line.startswith('Disjunct')


class Test(unittest.TestCase):

    def convert(self, cmd, type, **kwds):
        args=re.split('[ ]+',cmd)
        args.append("--symbolic-solver-labels") # for readability / quick inspections
        if 'file' in kwds:
            OUTPUT=kwds['file']
        else:
            OUTPUT=StringIO.StringIO()
        setup_redirect(OUTPUT)
        os.chdir(currdir)
        if type == 'lp':
            output = main.pyomo2lp(list(args))
        else:
            output = main.pyomo2nl(list(args))
        reset_redirect()
        if not 'file' in kwds:
            return OUTPUT.getvalue()
        return output

    # in the convert tests, make sure everything is generating symbolic solver labels - aids debugging.
    def run_convert2nl(self, cmd, file=None):
        return pyutilib.subprocess.run('pyomo2nl --symbolic-solver-labels '+cmd, outfile=file)

    def run_convert2lp(self, cmd, file=None):
        return pyutilib.subprocess.run('pyomo2lp --symbolic-solver-labels '+cmd, outfile=file)

    def test1a(self):
        """Simple execution of 'convert2nl'"""
        self.convert('pmedian.py pmedian.dat', type='nl', file=currdir+'test1a.out')
        self.assertFileEqualsBaseline(currdir+'unknown.nl', currdir+'_pmedian.nl')
        os.remove(currdir+'test1a.out')

    def test1b(self):
        """Simple execution of 'pyomo2nl'"""
        self.run_convert2nl('pmedian.py pmedian.dat', file=currdir+'test1b.out')
        self.assertFileEqualsBaseline(currdir+'unknown.nl', currdir+'_pmedian.nl')
        os.remove(currdir+'test1b.out')

    def test2a(self):
        """Simple execution of 'convert2lp'"""
        self.convert('pmedian.py pmedian.dat', type='lp', file=currdir+'test2a.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_pmedian.lp')
        os.remove(currdir+'test2a.out')

    def test2b(self):
        """Simple execution of 'pyomo2lp'"""
        self.run_convert2lp('pmedian.py pmedian.dat', file=currdir+'test2b.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_pmedian.lp')
        os.remove(currdir+'test2b.out')

    def test3a(self):
        """Simple execution of 'convert2nl'"""
        self.convert('pmedian4.py', type='nl', file=currdir+'test3a.out')
        self.assertFileEqualsBaseline(currdir+'unknown.nl', currdir+'_pmedian4.nl')
        os.remove(currdir+'test3a.out')

    def test3b(self):
        """Simple execution of 'pyomo2nl'"""
        self.run_convert2nl('pmedian4.py', file=currdir+'test3b.out')
        self.assertFileEqualsBaseline(currdir+'unknown.nl', currdir+'_pmedian4.nl')
        os.remove(currdir+'test3b.out')

    def test4a(self):
        """Simple execution of 'convert2lp' with a concrete model"""
        self.convert('pmedian4.py', type='lp', file=currdir+'test4a.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_pmedian4.lp')
        os.remove(currdir+'test4a.out')

    def test4b(self):
        """Simple execution of 'pyomo2lp' with a concrete model"""
        self.run_convert2lp('pmedian4.py', file=currdir+'test4b.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_pmedian4.lp')
        os.remove(currdir+'test4b.out')

    def test_quadratic1(self):
        """Test examples/pyomo/quadratic/example1.py"""
        self.run_convert2lp(os.path.join(scriptdir,'examples','pyomo','quadratic','example1.py'), file=currdir+'quadratic1.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_quadratic1.lp')
        os.remove(currdir+'quadratic1.out')

    def test_quadratic2(self):
        """Test examples/pyomo/quadratic/example2.py"""
        self.run_convert2lp(os.path.join(scriptdir,'examples','pyomo','quadratic','example2.py'), file=currdir+'quadratic2.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_quadratic2.lp')
        os.remove(currdir+'quadratic2.out')

    def test_quadratic3(self):
        """Test examples/pyomo/quadratic/example3.py"""
        self.run_convert2lp(os.path.join(scriptdir,'examples','pyomo','quadratic','example3.py'), file=currdir+'quadratic3.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_quadratic3.lp')
        os.remove(currdir+'quadratic3.out')

    def test_quadratic4(self):
        """Test examples/pyomo/quadratic/example4.py"""
        self.run_convert2lp(os.path.join(scriptdir,'examples','pyomo','quadratic','example4.py'), file=currdir+'quadratic4.out')
        self.assertFileEqualsBaseline(currdir+'unknown.lp', currdir+'_quadratic4.lp')
        os.remove(currdir+'quadratic4.out')


if __name__ == "__main__":
    unittest.main()



#
# Unit Tests for Set() Objects
#
# PyomoModel            Base test class
# SimpleSetA            Testing simple set of integers
# SimpleSetAordered     Testing simple set of ordered integers
# TestRangeSet          Testing the RangeSet class
# TestRangeSet1         More testing of the RangeSet class
# SimpleSetB            Testing simple set of string values
# SimpleSetC            Testing simple set of tuples
# ArraySet              Testing arrays of sets
# RealSetTests          Testing the RealSet class
# IntegerSetTests       Testing the IntegerSet class
# SetArgs1              Testing arguments for simple set
# SetArgs2              Testing arguments for arrays of sets
# Misc                  Misc tests
# SetIO                 Testing Set IO formats
#

import unittest
import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep
sys.path.insert(0, dirname(dirname(abspath(__file__)))+"/../..")
from coopr.pyomo import *
import coopr.pyomo.base
from coopr.pyomo.base.set_types import _AnySet


class PyomoModel(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class SimpleSetA(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 7; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.A = Set()
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,3,5,7])
        self.model.tmpset2 = Set(initialize=[1,2,3,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        self.model.setunion = Set(initialize=[1,2,3,5,7,9])
        self.model.setintersection = Set(initialize=[3,5,7])
        self.model.setxor = Set(initialize=[1,2,9])
        self.model.setdiff = Set(initialize=[1])
        self.model.setmul = Set(initialize=[(1,2), (1,3), (1,5), (1,7), (1,9),
                                      (3,2), (3,3), (3,5), (3,7), (3,9),
                                      (5,2), (5,3), (5,5), (5,7), (5,9),
                                      (7,2), (7,3), (7,5), (7,7), (7,9)])

        self.instance = self.model.create(currdir+"setA.dat")

        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        if os.path.exists(currdir+"setA.dat"):
            os.remove(currdir+"setA.dat")

    def test_len(self):
        """Check that a simple set of numeric elements has the right size"""
        self.assertEqual( len(self.instance.A), 4)

    def test_data(self):
        """Check that we can access the underlying set data"""
        self.assertEqual( len(self.instance.A.data()), 4)

    def test_dim(self):
        """Check that a simple set has dimension zero for its indexing"""
        self.assertEqual( self.instance.A.dim(), 0)

    def test_clear(self):
        """Check the clear() method empties the set"""
        self.instance.A.clear()
        self.assertEqual( len(self.instance.A), 0)

    def test_virtual(self):
        """Check if this is not a virtual set"""
        self.assertEqual( self.instance.A.virtual, False)

    def test_bounds(self):
        """Verify the bounds on this set"""
        self.assertEqual( self.instance.A.bounds(), (1,7))

    def test_check_values(self):
        """Check if the values added to this set are valid"""
        #
        # This should not throw an exception here
        #
        self.instance.A.check_values()

    def test_first(self):
        """Check that we can get the 'first' value in the set"""
        self.tmp = self.instance.A.first()
        self.assertNotEqual( self.tmp, None )

    def test_addValid(self):
        """Check that we can add valid set elements"""
        self.instance.A.add(self.e2,self.e4)
        self.assertEqual( len(self.instance.A), 6)
        self.assertFalse( self.e2 not in self.instance.A, "Cannot find new element in A")
        self.assertFalse( self.e4 not in self.instance.A, "Cannot find new element in A")

    def test_addInvalid(self):
        """Check that we get an error when adding invalid set elements"""
        #
        # This verifies that by default, all set elements are valid.  That
        # is, the default within is None
        #
        self.assertEqual( self.instance.A.domain, None)
        self.instance.A.add('2','3','4')
        self.assertFalse( '2' not in self.instance.A, "Found invalid new element in A")

    def test_removeValid(self):
        """Check that we can remove a valid set element"""
        self.instance.A.remove(self.e3)
        self.assertEqual( len(self.instance.A), 3)
        self.assertFalse( 3 in self.instance.A, "Found element in A that we removed")

    def test_removeInvalid(self):
        """Check that we fail to remove an invalid set element"""
        self.assertRaises(KeyError, self.instance.A.remove, 2)
        self.assertEqual( len(self.instance.A), 4)

    def test_discardValid(self):
        """Check that we can discard a valid set element"""
        self.instance.A.discard(self.e3)
        self.assertEqual( len(self.instance.A), 3)
        self.assertFalse( 3 in self.instance.A, "Found element in A that we removed")

    def test_discardInvalid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        self.instance.A.discard(self.e2)
        self.assertEqual( len(self.instance.A), 4)

    def test_iterator(self):
        """Check that we can iterate through the set"""
        self.tmp = set()
        for val in self.instance.A:
            self.tmp.add(val)
        self.assertFalse( self.tmp != self.instance.A.data(), "Set values found by the iterator appear to be different from the underlying set")

    def test_eq1(self):
        """Various checks for set equality and inequality (1)"""
        self.assertEqual( self.instance.A == self.instance.tmpset1, True)
        self.assertEqual( self.instance.tmpset1 == self.instance.A, True)
        self.assertEqual( self.instance.A != self.instance.tmpset1, False)
        self.assertEqual( self.instance.tmpset1 != self.instance.A, False)

    def test_eq2(self):
        """Various checks for set equality and inequality (2)"""
        self.assertEqual( self.instance.A == self.instance.tmpset2, False)
        self.assertEqual( self.instance.tmpset2 == self.instance.A, False)
        self.assertEqual( self.instance.A != self.instance.tmpset2, True)
        self.assertEqual( self.instance.tmpset2 != self.instance.A, True)

    def test_le1(self):
        """Various checks for set subset (1)"""
        self.assertEqual( self.instance.A < self.instance.tmpset1, False)
        self.assertEqual( self.instance.A <= self.instance.tmpset1, True)
        self.assertEqual( self.instance.A > self.instance.tmpset1, False)
        self.assertEqual( self.instance.A >= self.instance.tmpset1, True)
        self.assertEqual( self.instance.tmpset1 < self.instance.A, False)
        self.assertEqual( self.instance.tmpset1 <= self.instance.A, True)
        self.assertEqual( self.instance.tmpset1 > self.instance.A, False)
        self.assertEqual( self.instance.tmpset1 >= self.instance.A, True)

    def test_le2(self):
        """Various checks for set subset (2)"""
        self.assertEqual( self.instance.A < self.instance.tmpset2, True)
        self.assertEqual( self.instance.A <= self.instance.tmpset2, True)
        self.assertEqual( self.instance.A > self.instance.tmpset2, False)
        self.assertEqual( self.instance.A >= self.instance.tmpset2, False)
        self.assertEqual( self.instance.tmpset2 < self.instance.A, False)
        self.assertEqual( self.instance.tmpset2 <= self.instance.A, False)
        self.assertEqual( self.instance.tmpset2 > self.instance.A, True)
        self.assertEqual( self.instance.tmpset2 >= self.instance.A, True)

    def test_le3(self):
        """Various checks for set subset (3)"""
        self.assertEqual( self.instance.A < self.instance.tmpset3, False)
        self.assertEqual( self.instance.A <= self.instance.tmpset3, False)
        self.assertEqual( self.instance.A > self.instance.tmpset3, False)
        self.assertEqual( self.instance.A >= self.instance.tmpset3, False)
        self.assertEqual( self.instance.tmpset3 < self.instance.A, False)
        self.assertEqual( self.instance.tmpset3 <= self.instance.A, False)
        self.assertEqual( self.instance.tmpset3 > self.instance.A, False)
        self.assertEqual( self.instance.tmpset3 >= self.instance.A, False)

    def test_contains(self):
        """Various checks for contains() method"""
        self.assertEqual( self.e1 in self.instance.A, True)
        self.assertEqual( self.e2 in self.instance.A, False)
        self.assertEqual( '2' in self.instance.A, False)

    def test_or(self):
        """Check that set union works"""
        self.instance.tmp = self.instance.A | self.instance.tmpset3
        self.instance.tmp.construct()
        self.assertEqual( self.instance.tmp == self.instance.setunion, True)

    def test_and(self):
        """Check that set intersection works"""
        self.instance.tmp = self.instance.A & self.instance.tmpset3
        self.instance.tmp.construct()
        #print self.instance.tmp.data()
        #print self.instance.setintersection.data()
        self.assertEqual( self.instance.tmp == self.instance.setintersection, True)

    def test_xor(self):
        """Check that set exclusive or works"""
        self.instance.tmp = self.instance.A ^ self.instance.tmpset3
        self.instance.tmp.construct()
        self.assertEqual( self.instance.tmp == self.instance.setxor, True)

    def test_diff(self):
        """Check that set difference works"""
        self.instance.tmp = self.instance.A - self.instance.tmpset3
        self.instance.tmp.construct()
        self.assertEqual( self.instance.tmp == self.instance.setdiff, True)

    def test_mul(self):
        """Check that set cross-product works"""
        self.instance.tmp = self.instance.A * self.instance.tmpset3
        self.instance.tmp.construct()
        self.assertEqual( self.instance.tmp == self.instance.setmul, True)

    def test_filter_constructor(self):
        """ Check that sets can filter out unwanted elements """
        def evenFilter(model, el):
            return el % 2 == 0
        self.instance.tmp = Set(initialize=xrange(0,10), filter=evenFilter)
        self.instance.tmp.construct()
        self.assertEqual(sorted([x for x in self.instance.tmp]) == [0,2,4,6,8], True)

    def test_filter_attribute(self):
        """ Check that sets can filter out unwanted elements """
        def evenFilter(model, el):
            return el % 2 == 0
        self.instance.tmp = Set(initialize=xrange(0,10))
        self.instance.tmp.filter = evenFilter
        self.instance.tmp.construct()
        self.assertEqual(sorted([x for x in self.instance.tmp]) == [0,2,4,6,8], True)


class SimpleSetAordered(SimpleSetA):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 7; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.A = Set(ordered=True)
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,3,5,7])
        self.model.tmpset2 = Set(initialize=[1,2,3,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        self.model.setunion = Set(initialize=[1,2,3,5,7,9])
        self.model.setintersection = Set(initialize=[3,5,7])
        self.model.setxor = Set(initialize=[1,2,9])
        self.model.setdiff = Set(initialize=[1])
        self.model.setmul = Set(initialize=[(1,2), (1,3), (1,5), (1,7), (1,9),
                                      (3,2), (3,3), (3,5), (3,7), (3,9),
                                      (5,2), (5,3), (5,5), (5,7), (5,9),
                                      (7,2), (7,3), (7,5), (7,7), (7,9)])

        self.instance = self.model.create(currdir+"setA.dat")

        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6

    def test_first(self):
        """Check that we can get the 'first' value in the set"""
        self.tmp = self.instance.A.first()
        self.assertNotEqual( self.tmp, None )
        self.assertEqual( self.tmp, 1 )

    def test_ordered(self):
        tmp=[]
        for val in self.instance.A:
            tmp.append(val)
        self.assertEqual( tmp, [1,3,5,7] )


class TestRangeSet(SimpleSetA):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.A = RangeSet(1,5)
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,2,3,4,5])
        self.model.tmpset2 = Set(initialize=[1,2,3,4,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        self.model.setunion = Set(initialize=[1,2,3,4,5,7,9])
        self.model.setintersection = Set(initialize=[2,3,5])
        self.model.setxor = Set(initialize=[1,4,7,9])
        self.model.setdiff = Set(initialize=[1,4])
        self.model.setmul = Set(initialize=[(1,2), (1,3), (1,5), (1,7), (1,9),
                                      (2,2), (2,3), (2,5), (2,7), (2,9),
                                      (3,2), (3,3), (3,5), (3,7), (3,9),
                                      (4,2), (4,3), (4,5), (4,7), (4,9),
                                      (5,2), (5,3), (5,5), (5,7), (5,9)])

        self.instance = self.model.create()

        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6

    def test_member(self):
        self.assertEqual(self.instance.A.member(-1),5)
        self.assertEqual(self.instance.A.member(-2),4)
        self.assertEqual(self.instance.A.member(-3),3)
        self.assertEqual(self.instance.A.member(-4),2)
        self.assertEqual(self.instance.A.member(-5),1)
        self.assertEqual(self.instance.A.member(1),1)
        self.assertEqual(self.instance.A.member(2),2)
        self.assertEqual(self.instance.A.member(3),3)
        self.assertEqual(self.instance.A.member(4),4)
        self.assertEqual(self.instance.A.member(5),5)
        try:
            self.instance.A.member(6)
            self.fail("Expected IndexError")
        except IndexError:
            pass
        try:
            self.instance.A.member(-6)
            self.fail("Expected IndexError")
        except IndexError:
            pass
        try:
            self.instance.A.member(0)
            self.fail("Expected IndexError")
        except IndexError:
            pass

    def test_clear(self):
        """Check the clear() method empties the set"""
        try:
            self.instance.A.clear()
            self.fail("Expected TypeError because a RangeSet is a virtual set")
        except TypeError:
            pass

    def test_virtual(self):
        """Check if this is a virtual set"""
        self.assertEqual( self.instance.A.virtual, True)

    def test_bounds(self):
        """Verify the bounds on this set"""
        self.assertEqual( self.instance.A.bounds(), (1,5))

    def test_addValid(self):
        """Check that we can add valid set elements"""
        pass

    def test_addInvalid(self):
        """Check that we get an error when adding invalid set elements"""
        #
        # This verifies that by default, all set elements are valid.  That
        # is, the default within is None
        #
        try:
            self.instance.A.add('2','3','4')
            self.fail("Expected to generate an error when we remove an element from a RangeSet")
        except TypeError:
            pass
        self.assertFalse( '2' in self.instance.A, "Value we attempted to add is not in A")

    def test_removeValid(self):
        """Check that we can remove a valid set element"""
        try:
            self.instance.A.remove(self.e3)
            self.fail("Expected to generate an error when we remove an element from a RangeSet")
        except KeyError:
            pass
        self.assertEqual( len(self.instance.A), 5)
        self.assertTrue( self.e3 in self.instance.A, "Element is still in A")

    def test_removeInvalid(self):
        """Check that we fail to remove an invalid set element"""
        self.assertRaises(KeyError, self.instance.A.remove, 6)
        self.assertEqual( len(self.instance.A), 5)

    def test_remove(self):
        """ Check that the elements are properly removed  by .remove """
        pass

    def test_discardValid(self):
        """Check that we can discard a valid set element"""
        try:
            self.instance.A.discard(self.e3)
            self.fail("Expected to generate an error when we discare an element from a RangeSet")
        except KeyError:
            pass
        self.assertEqual( len(self.instance.A), 5)
        self.assertTrue( self.e3 in self.instance.A, "Found element in A that attemped to discard")

    def test_discardInvalid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        pass

    def test_contains(self):
        """Various checks for contains() method"""
        self.assertEqual( self.e1 in self.instance.A, True)
        self.assertEqual( self.e2 in self.instance.A, True)
        self.assertEqual( '2' in self.instance.A, False)

    def test_len(self):
        """Check that a simple set of numeric elements has the right size"""
        self.assertEqual( len(self.instance.A), 5)

    def test_data(self):
        """Check that we can access the underlying set data"""
        self.assertEqual( len(self.instance.A.data()), 5)

    def test_filter_constructor(self):
        """ Check that RangeSets can filter out unwanted elements """
        def evenFilter(model, el):
            return el % 2 == 0
        self.instance.tmp = RangeSet(0,10, filter=evenFilter)
        self.instance.tmp.construct()
        self.assertEqual(sorted([x for x in self.instance.tmp]) == [0,2,4,6,8,10], True)

    def test_filter_attribute(self):
        """ Check that RangeSets can filter out unwanted elements """
        def evenFilter(model, el):
            return el % 2 == 0
        self.instance.tmp = RangeSet(0,10)
        self.instance.tmp.filter = evenFilter
        self.instance.tmp.construct()
        self.assertEqual(sorted([x for x in self.instance.tmp]) == [0,2,4,6,8,10], True)


class TestRangeSet2(TestRangeSet):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        def validate_fn(model, val):
            return (val >= 1) and (val <= 5)

        self.model.A = RangeSet(1,10, validate=validate_fn)
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,2,3,4,5])
        self.model.tmpset2 = Set(initialize=[1,2,3,4,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        self.model.setunion = Set(initialize=[1,2,3,4,5,7,9])
        self.model.setintersection = Set(initialize=[2,3,5])
        self.model.setxor = Set(initialize=[1,4,7,9])
        self.model.setdiff = Set(initialize=[1,4])
        self.model.setmul = Set(initialize=[(1,2), (1,3), (1,5), (1,7), (1,9),
                                      (2,2), (2,3), (2,5), (2,7), (2,9),
                                      (3,2), (3,3), (3,5), (3,7), (3,9),
                                      (4,2), (4,3), (4,5), (4,7), (4,9),
                                      (5,2), (5,3), (5,5), (5,7), (5,9)])

        self.instance = self.model.create()

        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6


class TestRangeSet3(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        self.model.A = RangeSet(1.0,5.0,0.8)
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,2,3,4,5])
        self.model.tmpset2 = Set(initialize=[1,2,3,4,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        self.model.setunion = Set(initialize=[1,2,3,4,5,7,9])
        self.model.setintersection = Set(initialize=[2,3,5])
        self.model.setxor = Set(initialize=[1,4,7,9])
        self.model.setdiff = Set(initialize=[1,4])
        self.model.setmul = Set(initialize=[(1,2), (1,3), (1,5), (1,7), (1,9),
                                      (2,2), (2,3), (2,5), (2,7), (2,9),
                                      (3,2), (3,3), (3,5), (3,7), (3,9),
                                      (4,2), (4,3), (4,5), (4,7), (4,9),
                                      (5,2), (5,3), (5,5), (5,7), (5,9)])

        self.instance = self.model.create()

        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6

    def test_bounds(self):
        """Verify the bounds on this set"""
        self.assertEqual( self.instance.A.bounds(), (1,5))



class TestRangeSetMisc(unittest.TestCase):

    def test_constructor1(self):
        a=RangeSet(10)
        a.construct()
        tmp=[]
        for i in a:
            tmp.append(i)
        self.assertEqual(tmp, range(1,11))
        self.assertEqual( a.bounds(), (1,10))


    def test_constructor2(self):
        a=RangeSet(1,10,2)
        a.construct()
        tmp=[]
        for i in a:
            tmp.append(i)
        self.assertEqual(tmp, range(1,11,2))
        self.assertEqual( a.bounds(), (1,9))

    def test_constructor3(self):
        model=AbstractModel()
        model.a=Param(initialize=1)
        model.b=Param(initialize=2)
        model.c=Param(initialize=10)
        model.d=RangeSet( model.a*model.a, model.c*model.a, model.a*model.b)
        instance=model.create()
        tmp=[]
        for i in instance.d:
            tmp.append(i)
        self.assertEqual(tmp, range(1,11,2))
        self.assertEqual( instance.d.bounds(), (1,9))

class SimpleSetB(SimpleSetA):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := A1 A3 A5 A7; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.A = Set()
        #
        # Debugging
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=['A1','A3','A5','A7'])
        self.model.tmpset2 = Set(initialize=['A1','A2','A3','A5','A7'])
        self.model.tmpset3 = Set(initialize=['A2','A3','A5','A7','A9'])

        self.model.setunion = Set(initialize=['A1','A2','A3','A5','A7','A9'])
        self.model.setintersection = Set(initialize=['A3','A5','A7'])
        self.model.setxor = Set(initialize=['A1','A2','A9'])
        self.model.setdiff = Set(initialize=['A1'])
        self.model.setmul = Set(initialize=[('A1','A2'), ('A1','A3'), ('A1','A5'), ('A1','A7'), ('A1','A9'),
                                      ('A3','A2'), ('A3','A3'), ('A3','A5'), ('A3','A7'), ('A3','A9'),
                                      ('A5','A2'), ('A5','A3'), ('A5','A5'), ('A5','A7'), ('A5','A9'),
                                      ('A7','A2'), ('A7','A3'), ('A7','A5'), ('A7','A7'), ('A7','A9')])

        self.instance = self.model.create(currdir+"setA.dat")
        self.e1='A1'
        self.e2='A2'
        self.e3='A3'
        self.e4='A4'
        self.e5='A5'
        self.e6='A6'

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), None)


class SimpleSetC(SimpleSetA):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := (A1,1) (A3,1) (A5,1) (A7,1); end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.A = Set(dimen=2)
        #
        # Debugging
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[('A1',1), ('A3',1), ('A5',1), ('A7',1)])
        self.model.tmpset2 = Set(initialize=[('A1',1),('A2',1),('A3',1),('A5',1),('A7',1)])
        self.model.tmpset3 = Set(initialize=[('A2',1),('A3',1),('A5',1),('A7',1),('A9',1)])

        self.model.setunion = Set(initialize=[('A1',1),('A2',1),('A3',1),('A5',1),('A7',1),('A9',1)])
        self.model.setintersection = Set(initialize=[('A3',1),('A5',1),('A7',1)])
        self.model.setxor = Set(initialize=[('A1',1),('A2',1),('A9',1)])
        self.model.setdiff = Set(initialize=[('A1',1)])
        self.model.setmul = Set(initialize=[(('A1',1,'A2',1)), (('A1',1,'A3',1)), (('A1',1,'A5',1)), (('A1',1,'A7',1)), (('A1',1,'A9',1)),
                                      (('A3',1,'A2',1)), (('A3',1,'A3',1)), (('A3',1,'A5',1)), (('A3',1,'A7',1)), (('A3',1,'A9',1)),
                                      (('A5',1,'A2',1)), (('A5',1,'A3',1)), (('A5',1,'A5',1)), (('A5',1,'A7',1)), (('A5',1,'A9',1)),
                                      (('A7',1,'A2',1)), (('A7',1,'A3',1)), (('A7',1,'A5',1)), (('A7',1,'A7',1)), (('A7',1,'A9',1))])

        self.instance = self.model.create(currdir+"setA.dat")
        self.e1=('A1',1)
        self.e2=('A2',1)
        self.e3=('A3',1)
        self.e4=('A4',1)
        self.e5=('A5',1)
        self.e6=('A6',1)

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        os.remove(currdir+"setA.dat")

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), None)

    def test_addInvalid(self):
        """Check that we get an error when adding invalid set elements"""
        #
        # This verifies that by default, all set elements are valid.  That
        # is, the default within is None
        #
        self.assertEqual( self.instance.A.domain, None)
        try:
            self.instance.A.add('2','3','4')
        except ValueError:
            pass
        else:
            self.fail("fail test_addInvalid")
        self.assertFalse( '2' in self.instance.A, "Found invalid new element in A")
        self.instance.A.add(('2','3'))


class ArraySet(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 7; set A[C] := 3 5 7 9; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z)
        #
        # Debugging
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set()
        self.model.tmpset2 = Set()
        self.model.tmpset3 = Set()

        self.model.S = RangeSet(0,5)
        self.model.T = RangeSet(0,5)
        self.model.R = RangeSet(0,3)
        self.model.Q_a = Set(initialize=[1,3,5,7])
        self.model.Q_c = Set(initialize=[3,5,7,9])

        self.instance = self.model.create(currdir+"setA.dat")
        self.e1=('A1',1)

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), None)

    def test_getitem(self):
        """Check the access to items"""
        try:
            tmp=[]
            for val in self.instance.A['A']:
                tmp.append(val)
            tmp.sort()
        except:
            self.fail("Problems getting a valid set from a set array")
        self.assertEqual( tmp, [1,3,5,7])
        try:
            tmp = self.instance.A['D']
        except KeyError:
            pass
        else:
            self.fail("Problems getting an invalid set from a set array")

    def test_setitem(self):
        """Check the access to items"""
        try:
            self.model.Z = Set(initialize=['A','C'])
            self.model.A = Set(self.model.Z,initialize={'A':[1]})
            self.instance = self.model.create()
            tmp=[1,6,9]
            self.instance.A['A'] = tmp
            self.instance.A['C'] = tmp
        except:
            self.fail("Problems setting a valid set into a set array")
        try:
            self.instance.A['D'] = tmp
        except KeyError:
            pass
        else:
            self.fail("Problems setting an invalid set into a set array")

    def test_keys(self):
        """Check the keys for the array"""
        tmp=self.instance.A.keys()
        tmp.sort()
        self.assertEqual(tmp, ['A', 'C'])

    def test_len(self):
        """Check that a simple set of numeric elements has the right size"""
        try:
            len(self.instance.A)
        except TypeError:
            self.fail("fail test_len")
        else:
            pass


    def test_data(self):
        """Check that we can access the underlying set data"""
        try:
            self.instance.A.data()
        except TypeError:
            pass
        else:
            self.fail("fail test_data")

    def test_dim(self):
        """Check that a simple set has dimension zero for its indexing"""
        self.assertEqual( self.instance.A.dim(), 1)

    def test_clear(self):
        """Check the clear() method empties the set"""
        self.instance.A.clear()
        for key in self.instance.A:
            self.assertEqual( len(self.instance.A[key]), 0)

    def test_virtual(self):
        """Check if this is not a virtual set"""
        self.assertEqual( self.instance.A.virtual, False)

    def test_check_values(self):
        """Check if the values added to this set are valid"""
        #
        # This should not throw an exception here
        #
        self.instance.A.check_values()

    def test_first(self):
        """Check that we can get the 'first' value in the set"""
        pass

    def test_removeValid(self):
        """Check that we can remove a valid set element"""
        pass

    def test_removeInvalid(self):
        """Check that we fail to remove an invalid set element"""
        pass

    def test_discardValid(self):
        """Check that we can discard a valid set element"""
        pass

    def test_discardInvalid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        pass

    def test_iterator(self):
        """Check that we can iterate through the set"""
        tmp = 0
        for key in self.instance.A:
            tmp += len(self.instance.A[key])
        self.assertEqual( tmp, 8)

    def test_eq1(self):
        """ Various checks for set equality and inequality (1) """
        self.assertEqual(self.instance.A != self.instance.tmpset1, True)
        self.assertEqual(self.instance.tmpset1 != self.instance.A, True)
        self.assertEqual(self.instance.A == self.instance.tmpset1, False)
        self.assertEqual(self.instance.tmpset1 == self.instance.A, False)

    def test_eq2(self):
        """ Various checks for set equality and inequality (2) """
        self.assertEqual(self.instance.A == self.instance.tmpset2, False)
        self.assertEqual(self.instance.tmpset2 == self.instance.A, False)
        self.assertEqual(self.instance.A != self.instance.tmpset2, True)
        self.assertEqual(self.instance.tmpset2 != self.instance.A, True)

    def test_eq3(self):
        """ Various checks for set equality and inequality (3) """

        # Each test should be done with the arguments on each side to check
        # for commutativity

        # Self-equality
        self.assertEqual(self.instance.S == self.instance.S, True)
        self.assertEqual(self.instance.S != self.instance.S, False)

        # Equivalent members
        self.assertEqual(self.instance.S == self.instance.T, True)
        self.assertEqual(self.instance.T == self.instance.S, True)

        # Subset/superset nonequality
        self.assertEqual(self.instance.S != self.instance.R, True)
        self.assertEqual(self.instance.R != self.instance.S, True)

        # Manually initialized (Q_a) v. data file initialized (A["A"]) equality
        self.assertEqual(self.instance.A["A"] == self.instance.Q_a, True)
        self.assertEqual(self.instance.Q_a == self.instance.A["A"], True)

        # Manually initialized (Q_c) v. data file initialized (A["C"]) equality
        self.assertEqual(self.instance.A["C"] == self.instance.Q_c, True)
        self.assertEqual(self.instance.Q_c == self.instance.A["C"], True)

        # Comparison between Set and non-Set objects
        self.assertEqual(self.instance.A == 1.0, False)
        self.assertEqual(1.0 == self.instance.A, False)

        # Comparison between Set and non-Set objects
        self.assertEqual(self.instance.A != 1.0, True)
        self.assertEqual(1.0 != self.instance.A, True)

    def test_contains(self):
        """Various checks for contains() method"""
        tmp = self.e1 in self.instance.A
        self.assertEqual( tmp, False )

    def test_or(self):
        """Check that set union works"""
        try:
            self.instance.A | self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_or")

    def test_and(self):
        """Check that set intersection works"""
        try:
            self.instance.tmp = self.instance.A & self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_and")

    def test_xor(self):
        """Check that set exclusive or works"""
        try:
            self.instance.A ^ self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_xor")

    def test_diff(self):
        """Check that set difference works"""
        try:
            self.instance.A - self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_diff")

    def test_mul(self):
        """Check that set cross-product works"""
        try:
            self.instance.A * self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_mul")


class ArraySet2(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set Y := 1 2 ; set A[A,1] := 1 3 5 7; set A[C,2] := 3 5 7 9; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.Z = Set()
        self.model.Y = Set()
        self.model.A = Set(self.model.Z,self.model.Y)
        #
        # Debugging
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set()
        self.model.tmpset2 = Set()
        self.model.tmpset3 = Set()

        self.instance = self.model.create(currdir+"setA.dat")
        self.e1=('A1',1)

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), None)

    def test_getitem(self):
        """Check the access to items"""
        try:
            tmp=[]
            for val in self.instance.A['A',1]:
                tmp.append(val)
            tmp.sort()
        except:
            self.fail("Problems getting a valid set from a set array")
        self.assertEqual( tmp, [1,3,5,7])
        try:
            tmp = self.instance.A['A',2]
        except KeyError:
            pass
        else:
            self.fail("Problems getting an invalid set from a set array")

    def Xtest_setitem(self):
        """Check the access to items"""
        try:
            self.model.Y = Set(initialize=[1,2])
            self.model.Z = Set(initialize=['A','C'])
            self.model.A = Set(self.model.Z,self.model.Y,initialize={'A':[1]})
            self.instance = self.model.create()
            tmp=[1,6,9]
            self.instance.A['A'] = tmp
            self.instance.A['C'] = tmp
        except:
            self.fail("Problems setting a valid set into a set array")
        try:
            self.instance.A['D'] = tmp
        except KeyError:
            pass
        else:
            self.fail("Problems setting an invalid set into a set array")

    def Xtest_keys(self):
        """Check the keys for the array"""
        tmp=self.instance.A.keys()
        tmp.sort()
        self.assertEqual(tmp, ['A', 'C'])

    def Xtest_len(self):
        """Check that a simple set of numeric elements has the right size"""
        try:
            len(self.instance.A)
        except TypeError:
            pass
        else:
            self.fail("fail test_len")

    def Xtest_data(self):
        """Check that we can access the underlying set data"""
        try:
            self.instance.A.data()
        except TypeError:
            pass
        else:
            self.fail("fail test_data")

    def Xtest_dim(self):
        """Check that a simple set has dimension zero for its indexing"""
        self.assertEqual( self.instance.A.dim(), 1)

    def Xtest_clear(self):
        """Check the clear() method empties the set"""
        self.instance.A.clear()
        for key in self.instance.A:
            self.assertEqual( len(self.instance.A[key]), 0)

    def Xtest_virtual(self):
        """Check if this is not a virtual set"""
        self.assertEqual( self.instance.A.virtual, False)

    def Xtest_check_values(self):
        """Check if the values added to this set are valid"""
        #
        # This should not throw an exception here
        #
        self.instance.A.check_values()

    def Xtest_first(self):
        """Check that we can get the 'first' value in the set"""
        pass

    def Xtest_removeValid(self):
        """Check that we can remove a valid set element"""
        pass

    def Xtest_removeInvalid(self):
        """Check that we fail to remove an invalid set element"""
        pass

    def Xtest_discardValid(self):
        """Check that we can discard a valid set element"""
        pass

    def Xtest_discardInvalid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        pass

    def Xtest_iterator(self):
        """Check that we can iterate through the set"""
        tmp = 0
        for key in self.instance.A:
            tmp += len(self.instance.A[key])
        self.assertEqual( tmp, 8)

    def Xtest_eq1(self):
        """Various checks for set equality and inequality (1)"""
        try:
            self.assertEqual( self.instance.A == self.instance.tmpset1, True)
            self.assertEqual( self.instance.tmpset1 == self.instance.A, True)
            self.assertEqual( self.instance.A != self.instance.tmpset1, False)
            self.assertEqual( self.instance.tmpset1 != self.instance.A, False)
        except TypeError:
            pass
        else:
            self.fail("fail test_eq1")

    def Xtest_eq2(self):
        """Various checks for set equality and inequality (2)"""
        try:
            self.assertEqual( self.instance.A == self.instance.tmpset2, False)
            self.assertEqual( self.instance.tmpset2 == self.instance.A, False)
            self.assertEqual( self.instance.A != self.instance.tmpset2, True)
            self.assertEqual( self.instance.tmpset2 != self.instance.A, True)
        except TypeError:
            pass
        else:
            self.fail("fail test_eq2")

    def Xtest_contains(self):
        """Various checks for contains() method"""
        tmp = self.e1 in self.instance.A
        self.assertEqual( tmp, False )

    def Xtest_or(self):
        """Check that set union works"""
        try:
            self.instance.A | self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_or")

    def Xtest_and(self):
        """Check that set intersection works"""
        try:
            self.instance.tmp = self.instance.A & self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_and")

    def Xtest_xor(self):
        """Check that set exclusive or works"""
        try:
            self.instance.A ^ self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_xor")

    def Xtest_diff(self):
        """Check that set difference works"""
        try:
            self.instance.A - self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_diff")

    def Xtest_mul(self):
        """Check that set cross-product works"""
        try:
            self.instance.A * self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("fail test_mul")


class RealSetTests(SimpleSetA):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        x = RealSet()
        x.concrete=True
        self.model.A = x
        x.concrete=False
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1.1,3.1,5.1,7.1])
        self.model.tmpset2 = Set(initialize=[1.1,2.1,3.1,5.1,7.1])
        self.model.tmpset3 = Set(initialize=[2.1,3.1,5.1,7.1,9.1])

        y = RealSet()
        y.concrete=True
        self.model.setunion = y
        y.concrete=False
        self.model.setintersection = Set(initialize=[1.1,3.1,5.1,7.1])
        self.model.setxor = Set(initialize=[])
        self.model.setdiff = Set(initialize=[])
        self.model.setmul = None

        self.instance = self.model.create()
        self.e1=1.1
        self.e2=2.1
        self.e3=3.1
        self.e4=4.1
        self.e5=5.1
        self.e6=6.1

    def tearDown(self):
        pass

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), (None,None))

    def test_len(self):
        """Check that the set has the right size"""
        try:
            len(self.instance.A)
        except ValueError:
            pass
        else:
            self.fail("test_len failure")

    def test_data(self):
        """Check that we can access the underlying set data"""
        try:
            self.instance.A.data()
        except TypeError:
            pass
        else:
            self.fail("test_data failure")

    def test_clear(self):
        """Check that the clear() method generates an exception"""
        self.assertRaises(TypeError, self.instance.A.clear)

    def test_virtual(self):
        """Check if this is not a virtual set"""
        self.assertEqual( self.instance.A.virtual, True)

    def test_discardValid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        self.assertRaises(KeyError, self.instance.A.discard, self.e2)

    def test_discardInvalid(self):
        """Check that we fail to remove an invalid set element without an exception"""
        pass

    def test_removeValid(self):
        """Check that we can remove a valid set element"""
        self.assertRaises(KeyError, self.instance.A.remove, self.e3)

    def test_removeInvalid(self):
        pass

    def test_addInvalid(self):
        """Check that we get an error when adding invalid set elements"""
        pass

    def test_first(self):
        """Check that we can get the 'first' value in the set"""
        self.assertRaises(TypeError,self.instance.A.first)

    def test_addValid(self):
        """Check that we can add valid set elements"""
        self.assertEqual( self.instance.A.domain, None)
        self.assertRaises(TypeError,self.instance.A.add,2)

    def test_iterator(self):
        """Check that we can iterate through the set"""
        try:
            for val in self.instance.A:
                tmp=val
        except TypeError:
            pass
        else:
            self.fail("test_iterator failure")

    def test_eq1(self):
        """Various checks for set equality and inequality (1)"""
        self.assertTrue(not(self.instance.A == self.instance.tmpset1))
        self.assertTrue(not(self.instance.tmpset1 == self.instance.A))
        self.assertTrue(self.instance.A != self.instance.tmpset1)
        self.assertTrue(self.instance.tmpset1 != self.instance.A)


    def test_eq2(self):
        """Various checks for set equality and inequality (2)"""
        self.assertTrue(not(self.instance.A == self.instance.tmpset2))
        self.assertTrue(not(self.instance.tmpset2 == self.instance.A))
        self.assertTrue(self.instance.A != self.instance.tmpset2)
        self.assertTrue(self.instance.tmpset2 != self.instance.A)

    def test_le1(self):
        """Various checks for set subset (1)"""
        try:
            self.instance.A < self.instance.tmpset1
            self.instance.A <= self.instance.tmpset1
            self.instance.A > self.instance.tmpset1
            self.instance.A >= self.instance.tmpset1
            self.instance.tmpset1 < self.instance.A
            self.instance.tmpset1 <= self.instance.A
            self.instance.tmpset1 > self.instance.A
            self.instance.tmpset1 >= self.instance.A
        except TypeError:
            pass
        else:
            self.fail("test_le1 failure")

    def test_le2(self):
        """Various checks for set subset (2)"""
        try:
            self.instance.A < self.instance.tmpset2
            self.instance.A <= self.instance.tmpset2
            self.instance.A > self.instance.tmpset2
            self.instance.A >= self.instance.tmpset2
            self.instance.tmpset2 < self.instance.A
            self.instance.tmpset2 <= self.instance.A
            self.instance.tmpset2 > self.instance.A
            self.instance.tmpset2 >= self.instance.A
        except TypeError:
            pass
        else:
            self.fail("test_le2 failure")

    def test_le3(self):
        """Various checks for set subset (3)"""
        try:
            self.instance.A < self.instance.tmpset3
            self.instance.A <= self.instance.tmpset3
            self.instance.A > self.instance.tmpset3
            self.instance.A >= self.instance.tmpset3
            self.instance.tmpset3 < self.instance.A
            self.instance.tmpset3 <= self.instance.A
            self.instance.tmpset3 > self.instance.A
            self.instance.tmpset3 >= self.instance.A
        except TypeError:
            pass
        else:
            self.fail("test_le3 failure")

    def test_contains(self):
        """Various checks for contains() method"""
        self.assertEqual( self.e1 in self.instance.A, True)
        self.assertEqual( self.e2 in self.instance.A, True)
        self.assertEqual( '2' in self.instance.A, False)

    def test_or(self):
        """Check that set union works"""
        try:
            self.instance.tmp = self.instance.A | self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("Operator __or__ should have failed.")

    def test_and(self):
        """Check that set intersection works"""
        try:
            self.instance.tmp = self.instance.A & self.instance.tmpset3
        except TypeError:
            pass
        else:
            self.fail("Operator __and__ should have failed.")

    def test_xor(self):
        """Check that set exclusive or works"""
        try:
            self.tmp = self.instance.A ^ self.instance.tmpset3
        except:
            pass
        else:
            self.fail("Operator __xor__ should have failed.")

    def test_diff(self):
        """Check that set difference works"""
        try:
            self.tmp = self.instance.A - self.instance.tmpset3
        except:
            pass
        else:
            self.fail("Operator __diff__ should have failed.")

    def test_mul(self):
        """Check that set cross-product works"""
        try:
            self.instance.tmp = self.instance.A * self.instance.tmpset3
        except:
            pass
        else:
            self.fail("Operator __mul__ should have failed.")


class IntegerSetTests(RealSetTests):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        x = IntegerSet()
        x.concrete=True
        self.model.A = x
        x.concrete=False
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,3,5,7])
        self.model.tmpset2 = Set(initialize=[1,2,3,5,7])
        self.model.tmpset3 = Set(initialize=[2,3,5,7,9])

        y = IntegerSet()
        y.concrete=True
        self.model.setunion = y
        y.concrete=False
        self.model.setintersection = Set(initialize=[1,3,5,7])
        self.model.setxor = Set(initialize=[])
        self.model.setdiff = Set(initialize=[])
        self.model.setmul = None

        self.instance = self.model.create()
        self.e1=1
        self.e2=2
        self.e3=3
        self.e4=4
        self.e5=5
        self.e6=6

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), (None,None))


class AnySetTests(RealSetTests):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        # Create model instance
        #
        x = _AnySet()
        x.concrete=True
        self.model.A = x
        x.concrete=False
        #
        # Misc datasets
        #
        self.model.tmpset1 = Set(initialize=[1,'3',5,7])
        self.model.tmpset2 = Set(initialize=[1,2,'3',5,7])
        self.model.tmpset3 = Set(initialize=[2,'3',5,7,9])

        y = _AnySet()
        y.concrete=True
        self.model.setunion = y
        y.concrete=False
        self.model.setintersection = Set(initialize=[1,'3',5,7])
        self.model.setxor = Set(initialize=[])
        self.model.setdiff = Set(initialize=[])
        self.model.setmul = None

        self.instance = self.model.create()
        self.e1=1
        self.e2=2
        self.e3='3'
        self.e4=4
        self.e5=5
        self.e6=6

    def test_bounds(self):
        self.assertEqual( self.instance.A.bounds(), None)

    def test_contains(self):
        """Various checks for contains() method"""
        self.assertEqual( self.e1 in self.instance.A, True)
        self.assertEqual( self.e2 in self.instance.A, True)
        self.assertEqual( '2' in self.instance.A, True)


class SetArgs1(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def test_initialize1(self):
        self.model.A = Set(initialize=[1,2,3,'A'])
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.A),4)

    def test_initialize2(self):
        self.model.A = Set(initialize=[(i,j) for i in range(0,3) for j in range(1,4) if (i+j)%2 == 0])
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.A),4)

    def test_initialize3(self):
        self.model.A = Set(initialize=((i,j) for i in range(0,3) for j in range(1,4) if (i+j)%2 == 0))
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.A),4)

    def test_initialize4(self):
        self.model.A = Set(initialize=range(0,4))
        def B_index(model):
            return (i for i in model.A if i%2 == 0)
        def B_init(model, i):
            return range(i,2+i)
        self.model.B = Set(B_index, initialize=B_init)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(self.instance.B[0].data(),set([0,1]))
        self.assertEqual(self.instance.B[2].data(),set([2,3]))
        self.assertEqual(self.instance.B.keys(),[0,2])

    def test_initialize5(self):
        self.model.A = Set(initialize=range(0,4))
        def B_index(model):
            for i in model.A:
                if i%2 == 0:
                    yield i
        def B_init(model, i):
            return range(i,2+i)
        self.model.B = Set(B_index, initialize=B_init)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(self.instance.B[0].data(),set([0,1]))
        self.assertEqual(self.instance.B[2].data(),set([2,3]))
        self.assertEqual(self.instance.B.keys(),[0,2])

    def test_initialize6(self):
        self.model.A = Set(initialize=range(0,4))
        def B_index(model):
            for i in model.A:
                if i%2 == 0:
                    yield i
        def B_init(model, i, j):
            k=i+j               # A dummy calculation
            if j:
                return range(i,2+i)
            return []
        self.model.B = Set(B_index, [True,False], initialize=B_init)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(set(self.instance.B.keys()),set([(0,True),(2,True),(0,False),(2,False)]))
        self.assertEqual(self.instance.B[0,True].data(),set([0,1]))
        self.assertEqual(self.instance.B[2,True].data(),set([2,3]))

    def test_initialize7(self):
        self.model.A = Set(initialize=range(0,3))
        @set_options(dimen=3)
        def B_index(model):
            return [(i,i+1,i*i) for i in model.A]
        def B_init(model, i, ii, iii, j):
            k=i+j               # A dummy calculation
            if j:
                return range(i,2+i)
            return []
        self.model.B = Set(B_index, [True,False], initialize=B_init)
        self.instance = self.model.create()
        #self.instance.pprint()
        self.assertEqual(set(self.instance.B.keys()),set([(0,1,0,True),(1,2,1,True),(2,3,4,True),(0,1,0,False),(1,2,1,False),(2,3,4,False)]))
        self.assertEqual(self.instance.B[0,1,0,True].data(),set([0,1]))
        self.assertEqual(self.instance.B[2,3,4,True].data(),set([2,3]))

    def test_initialize8(self):
        self.model.A = Set(initialize=range(0,3))
        def B_index(model):
            return [(i,i+1,i*i) for i in model.A]
        def B_init(model, i, ii, iii, j):
            if j:
                return range(i,2+i)
            return []
        self.model.B = Set(B_index, [True,False], initialize=B_init)
        try:
            self.instance = self.model.create()
            self.fail("Expected ValueError because B_index returns a tuple")
        except ValueError:
            pass

    def test_initialize9(self):
        self.model.A = Set(initialize=range(0,3))
        @set_options(domain=Integers)
        def B_index(model):
            return [i/2.0 for i in model.A]
        def B_init(model, i, j):
            if j:
                return range(int(i),int(2+i))
            return []
        self.model.B = Set(B_index, [True,False], initialize=B_init)
        self.instance = self.model.create()
        try:
            self.instance = self.model.create()
            self.fail("Expected ValueError because B_index returns invalid set values")
        except AssertionError:
            pass

    def test_dimen1(self):
        #
        # Create model instance
        #
        self.model.A = Set(initialize=[1,2,3], dimen=1)
        self.instance = self.model.create()
        #
        try:
            self.model.A = Set(initialize=[4,5,6], dimen=2)
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("test_dimen")
        #
        self.model.A = Set(initialize=[(1,2), (2,3), (3,4)], dimen=2)
        self.instance = self.model.create()
        #
        try:
            self.model.A = Set(initialize=[(1,2), (2,3), (3,4)], dimen=1)
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("test_dimen")
        #
        def f(model):
            return [(1,1), (2,2), (3,3)]
        self.model.A = Set(initialize=f, dimen=2)
        self.instance = self.model.create()
        #
        try:
            self.model.A = Set(initialize=f, dimen=3)
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("test_dimen")

    def test_rule(self):
        #
        # Create model instance
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; end;"
        OUTPUT.close()
        def tmp_init(model):
            ##model.n.pprint()
            ##print "HERE",model.n,value(model.n)
            return range(0,value(model.n))
        self.model.n = Param()
        self.model.A = Set(initialize=tmp_init)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A),5)

    def test_rule2(self):
        #
        # Create model instance
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; end;"
        OUTPUT.close()
        @simple_set_rule
        def tmp_init(model, z):
            if z>value(model.n) or z == 11:
                return None
            return z
        self.model.n = Param()
        self.model.A = Set(initialize=tmp_init)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A),5)

    def test_rule3(self):
        #
        # Create model instance
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; end;"
        OUTPUT.close()
        def tmp_init(model, z):
            if z>value(model.n) or z == 11:
                return Set.End
            return z
        self.model.n = Param()
        self.model.A = Set(initialize=tmp_init)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A),5)

    def test_within1(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.A = Set(within=Integers)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_within1")

    def test_within2(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.A = Set(within=Reals)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            self.fail("fail test_within2")
        else:
            pass

    def test_validation1(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.A = Set(validate=lambda model, x:x<6)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_validation1")

    def test_validation2(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := 1 3 5 5.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.A = Set(validate=lambda model, x:x<6)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            self.fail("fail test_validation2")
        else:
            pass

    def test_other1(self):
        self.model.A = Set(initialize=[1,2,3,'A'], validate=lambda model, x:x in Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other2(self):
        self.model.A = Set(initialize=[1,2,3,'A'], within=Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other3(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; end;"
        OUTPUT.close()
        def tmp_init(model):
            tmp=[]
            for i in range(0,value(model.n)):
                tmp.append(i/2.0)
            return tmp
        self.model.n = Param()
        self.model.A = Set(initialize=tmp_init, validate=lambda model, x:x in Integers)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other4(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; end;"
        OUTPUT.close()
        def tmp_init(model):
            tmp=[]
            for i in range(0,value(model.n)):
                tmp.append(i/2.0)
            return tmp
        self.model.n = Param()
        self.model.A = Set(initialize=tmp_init, within=Integers)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        if os.path.exists(currdir+"setA.dat"):
            os.remove(currdir+"setA.dat")


class SetArgs2(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def test_initialize(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 7; end;"
        OUTPUT.close()
        #
        # Create model instance
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, initialize={'A':[1,2,3,'A']})
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.A['A']),4)

    def test_dimen(self):
        #
        # Create model instance
        #
        self.model.Z = Set(initialize=[1,2])
        self.model.A = Set(self.model.Z, initialize=[1,2,3], dimen=1)
        self.instance = self.model.create()
        try:
            self.model.A = Set(self.model.Z, initialize=[4,5,6], dimen=2)
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("test_dimen")
        self.model.A = Set(self.model.Z, initialize=[(1,2), (2,3), (3,4)], dimen=2)
        self.instance = self.model.create()
        try:
            self.model.A = Set(self.model.Z, initialize=[(1,2), (2,3), (3,4)], dimen=1)
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("test_dimen")

    def test_rule(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; set Z := A C; end;"
        OUTPUT.close()
        def tmp_init(model, i):
            return range(0,value(model.n))
        self.model.n = Param()
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, initialize=tmp_init)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A['A']),5)

    def test_rule2(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; set Z := A C; end;"
        OUTPUT.close()
        @simple_set_rule
        def tmp_rule2(model, z, i):
            if z>value(model.n):
                return None
            return z
        self.model.n = Param()
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, initialize=tmp_rule2)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A['A']),5)

    def test_rule3(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; param n := 5; set Z := A C; end;"
        OUTPUT.close()
        def tmp_rule2(model, z, i):
            if z>value(model.n):
                return Set.End
            return z
        self.model.n = Param()
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, initialize=tmp_rule2)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual(len(self.instance.A['A']),5)

    def test_within1(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, within=Integers)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_within1")

    def test_within2(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, within=Reals)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            self.fail("fail test_within2")
        else:
            pass

    def test_validation1(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 7.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, validate=lambda model, x:x<6)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            pass
        else:
            self.fail("fail test_within1")

    def test_validation2(self):
        #
        # Create Set 'A' data file
        #
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set Z := A C; set A[A] := 1 3 5 5.5; end;"
        OUTPUT.close()
        #
        # Create A with an error
        #
        self.model.Z = Set()
        self.model.A = Set(self.model.Z, validate=lambda model, x:x<6)
        try:
            self.instance = self.model.create(currdir+"setA.dat")
        except ValueError:
            self.fail("fail test_within2")
        else:
            pass

    def test_other1(self):
        self.model.Z = Set(initialize=['A'])
        self.model.A = Set(self.model.Z, initialize={'A':[1,2,3,'A']}, validate=lambda model, x:x in Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other2(self):
        self.model.Z = Set(initialize=['A'])
        self.model.A = Set(self.model.Z, initialize={'A':[1,2,3,'A']}, within=Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other3(self):
        def tmp_init(model, i):
            tmp=[]
            for i in range(0,value(model.n)):
                tmp.append(i/2.0)
            return tmp
        self.model.n = Param(initialize=5)
        self.model.Z = Set(initialize=['A'])
        self.model.A = Set(self.model.Z,initialize=tmp_init, validate=lambda model, x:x in Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def test_other4(self):
        def tmp_init(model, i):
            tmp=[]
            for i in range(0,value(model.n)):
                tmp.append(i/2.0)
            return tmp
        self.model.n = Param(initialize=5)
        self.model.Z = Set(initialize=['A'])
        self.model.A = Set(self.model.Z, initialize=tmp_init, within=Integers)
        self.model.B = Set(self.model.Z, rule=tmp_init, within=Integers)
        try:
            self.instance = self.model.create()
        except ValueError:
            pass
        else:
            self.fail("fail test_other1")

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        if os.path.exists(currdir+"setA.dat"):
            os.remove(currdir+"setA.dat")


class Misc(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        #
        #
        #
        self.model.A = Set(initialize=[1,2,3])
        self.model.B = Set(initialize=['a','b','c'])
        self.model.C = Set(initialize=[4,5,6])

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        if os.path.exists(currdir+"setA.dat"):
            os.remove(currdir+"setA.dat")

    def test_cross_set(self):
        self.model.C = self.model.A * self.model.B
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.C),9)

    def test_tricross_set(self):
        self.model.D = self.model.A * self.model.B * self.model.C
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.D),27)

    def test_virtual_cross_set(self):
        self.model.C = self.model.A * self.model.B
        self.model.C.virtual = True
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.C),9)
        if not self.instance.C.value is None:
            self.assertEqual(len(self.instance.C.value),0)
        tmp=[]
        for item in self.instance.C:
            tmp.append(item)
        self.assertEqual(len(tmp),9)


class SetIO(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        #
        # Remove Set 'A' data file
        #
        if os.path.exists(currdir+"setA.dat"):
            os.remove(currdir+"setA.dat")

    def test_io1(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set A := A1 A2 A3; end;"
        OUTPUT.close()
        self.model.A = Set()
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.A), 3)

    def test_io2(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data; set B := 1 2 3 4; end;"
        OUTPUT.close()
        self.model.B = Set()
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.B), 4)

    def test_io3(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := A1 A2 A3;"
        print >>OUTPUT, "set B := 1 2 3 4;"
        print >>OUTPUT, "set C := (A1,1) (A2,2) (A3,3);"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.B = Set()
        self.model.C = self.model.A * self.model.B
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.C), 12)

    def test_io4(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := A1 A2 A3;"
        print >>OUTPUT, "set B := 1 2 3 4;"
        print >>OUTPUT, "set D := (A1,1) (A2,2) (A3,3);"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.B = Set()
        self.model.D = Set(within=self.model.A*self.model.B)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.D), 3)

    def test_io5(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := A1 A2 A3;"
        print >>OUTPUT, "set B := 1 2 3 4;"
        print >>OUTPUT, "set D : A1 A2 A3 :="
        print >>OUTPUT, "    1   +  -  +"
        print >>OUTPUT, "    2   +  -  +"
        print >>OUTPUT, "    3   +  -  +"
        print >>OUTPUT, "    4   +  -  +;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.B = Set()
        self.model.D = Set(within=self.model.A*self.model.B)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.D), 8)

    def test_io6(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := A1 A2 A3;"
        print >>OUTPUT, "set B := 1 2 3 4;"
        print >>OUTPUT, "set E :="
        print >>OUTPUT, "(A1,1,*) A1 A2"
        print >>OUTPUT, "(A2,2,*) A2 A3"
        print >>OUTPUT, "(A3,3,*) A1 A3 ;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.B = Set()
        self.model.E = Set(within=self.model.A*self.model.B*self.model.A)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.E), 6)

    def test_io7(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := A1 A2 A3;"
        print >>OUTPUT, "set B := 1 2 3 4;"
        print >>OUTPUT, "set F[A1] := 1 3 5;"
        print >>OUTPUT, "set F[A2] := 2 4 6;"
        print >>OUTPUT, "set F[A3] := 3 5 7;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.B = Set()
        self.model.F = Set(self.model.A)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( self.instance.F.dim(), 1)
        self.assertEqual( len(self.instance.F.keys()), 3)
        self.assertEqual( len(self.instance.F['A1']), 3)

    def test_io8(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set E :="
        print >>OUTPUT, "(A1,1,*) A1 A2"
        print >>OUTPUT, "(*,2,*) A2 A3"
        print >>OUTPUT, "(A3,3,*) A1 A3 ;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.E = Set(dimen=3)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.E), 5)

    def test_io9(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set E :="
        print >>OUTPUT, "(A1,1,A1) (A1,1,A2)"
        print >>OUTPUT, "(A2,2,A3)"
        print >>OUTPUT, "(A3,3,A1) (A3,3,A3) ;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.E = Set(dimen=3)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( len(self.instance.E), 5)

    def test_io10(self):
        OUTPUT=open(currdir+"setA.dat","w")
        print >>OUTPUT, "data;"
        print >>OUTPUT, "set A := 'A1 x' ' A2' \"A3\";"
        print >>OUTPUT, "set F['A1 x'] := 1 3 5;"
        print >>OUTPUT, "set F[\" A2\"] := 2 4 6;"
        print >>OUTPUT, "set F['A3'] := 3 5 7;"
        print >>OUTPUT, "end;"
        OUTPUT.close()
        self.model.A = Set()
        self.model.F = Set(self.model.A)
        self.instance = self.model.create(currdir+"setA.dat")
        self.assertEqual( self.instance.F.dim(), 1)
        self.assertEqual( len(self.instance.F.keys()), 3)
        self.assertEqual( len(self.instance.F['A1 x']), 3)


def init_fn(model):
    return []

def tmp_constructor(model, ctr, index):
    if ctr == 10:
        return None
    else:
        return ctr

class SetErrors(PyomoModel):

    def test_membership(self):
        self.assertEqual( 0 in Boolean, True)
        self.assertEqual( 1 in Boolean, True)
        self.assertEqual( True in Boolean, True)
        self.assertEqual( False in Boolean, True)
        self.assertEqual( 1.1 in Boolean, False)
        self.assertEqual( 2 in Boolean, False)

        self.assertEqual( 0 in Integers, True)
        self.assertEqual( 1 in Integers, True)
        self.assertEqual( True in Integers, False)
        self.assertEqual( False in Integers, False)
        self.assertEqual( 1.1 in Integers, False)
        self.assertEqual( 2 in Integers, True)

        self.assertEqual( 0 in Reals, True)
        self.assertEqual( 1 in Reals, True)
        self.assertEqual( True in Reals, False)
        self.assertEqual( False in Reals, False)
        self.assertEqual( 1.1 in Reals, True)
        self.assertEqual( 2 in Reals, True)

        self.assertEqual( 0 in Any, True)
        self.assertEqual( 1 in Any, True)
        self.assertEqual( True in Any, True)
        self.assertEqual( False in Any, True)
        self.assertEqual( 1.1 in Any, True)
        self.assertEqual( 2 in Any, True)

    def test_setargs1(self):
        try:
            a=Set()
            c=Set(a,foo=None)
            self.fail("test_setargs1 - expected error because of bad argument")
        except ValueError:
            pass

    def test_setargs2(self):
        try:
            a=Set()
            b=Set(a)
            c=Set(within=b, dimen=2)
            self.fail("test_setargs1 - expected error because of bad argument")
        except ValueError:
            pass
        a=Set()
        b=Set(a)
        c=Set(within=b, dimen=1)
        self.assertEqual(c.domain,b)
        c.domain = a
        self.assertEqual(c.domain,a)

    def test_setargs3(self):
        a=Set(dimen=2, initialize=(1,2))
        try:
            a=Set(dimen=1, initialize=(1,2))
            self.fail("test_setargs3 - expected error because of bad argument")
        except ValueError:
            pass

    def test_verify(self):
        a=Set(initialize=[1,2,3])
        b=Set(within=a)
        try:
            b._verify(4)
            self.fail("test_verify - bad value was expected")
        except ValueError:
            pass
        self.assertEqual(b._verify(4,False),False)
        #
        c=Set()
        try:
            c._verify( (1,2) )
            self.fail("test_verify - bad value was expected")
        except ValueError:
            pass
        self.assertEqual(c._verify( (1,2),False),False)
        #
        c=Set(dimen=2)
        try:
            c._verify( (1,2,3) )
            self.fail("test_verify - bad value was expected")
        except ValueError:
            pass
        self.assertEqual(c._verify( (1,2,3),False),False)

    def test_construct(self):
        a = Set(initialize={})
        try:
            a.construct()
            self.fail("test_construct - expected failure constructing with a dictionary")
        except ValueError:
            pass
        #
        a = Set(initialize=init_fn)
        try:
            a.construct()
            self.fail("test_construct - expected exception due to None model")
        except ValueError:
            pass

    def test_add(self):
        a=Set()
        a.add(1)
        a.add("a")
        try:
            a.add({})
            self.fail("test_add - expected type error because {} is unhashable")
        except:
            pass

    def test_getitem(self):
        a=Set(initialize=[1,2])
        try:
            a[0]
            self.fail("test_getitem - cannot index an unordered set")
        except IndexError:
            pass

    def test_eq(self):
        a=Set(dimen=1,name="a",initialize=[1,2])
        a.construct()
        b=Set(dimen=2)
        self.assertEqual(a==b,False)
        self.assertTrue(not a.__eq__(Boolean))
        self.assertTrue(not Boolean.__eq__(a))

    def test_neq(self):
        a=Set(dimen=1,initialize=[1,2])
        a.construct()
        b=Set(dimen=2)
        self.assertEqual(a!=b,True)
        self.assertTrue(a.__ne__(Boolean))
        self.assertTrue(Boolean.__ne__(a))

    def test_contains(self):
        a=Set(initialize=[1,3,5,7])
        a.construct()
        b=Set(initialize=[1,3])
        b.construct()
        self.assertEqual(b in a, True)
        self.assertEqual(a in b, False)
        self.assertEqual(1 in Integers, True)
        self.assertEqual(1 in NonNegativeIntegers, True)

    def test_subset(self):
        try:
            Integers in Reals
            #Integers.issubset(Reals)
            self.fail("test_subset - expected TypeError")
        except TypeError:
            pass
        try:
            a=Set(dimen=1)
            b=Set(dimen=2)
            a in b
            self.fail("test_subset - expected ValueError")
        except ValueError:
            pass

    def test_superset(self):
        try:
            Reals >= Integers
            #Integers.issubset(Reals)
            self.fail("test_subset - expected TypeError")
        except TypeError:
            pass
        a=Set(initialize=[1,3,5,7])
        a.construct()
        b=Set(initialize=[1,3])
        b.construct()
        self.assertEqual(Reals >= b, True)
        self.assertEqual(Reals >= [1,3,7], True)
        self.assertEqual(Reals >= [1,3,7,"a"], False)
        self.assertEqual(a >= b, True)

    def test_lt(self):
        try:
            Integers < Reals
            self.fail("test_subset - expected TypeError")
        except TypeError:
            pass
        a=Set(initialize=[1,3,5,7])
        a.construct()
        a < Reals
        b=Set(initialize=[1,3,5])
        b.construct()
        self.assertEqual(a<a,False)
        self.assertEqual(b<a,True)
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            a<c
            self.fail("test_subset - expected ValueError")
        except ValueError:
            pass

    def test_gt(self):
        a=Set(initialize=[1,3,5,7])
        a.construct()
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            a>c
            self.fail("test_subset - expected ValueError")
        except ValueError:
            pass

    def test_or(self):
        a=Set(initialize=[1,2,3])
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            Reals | Integers
            self.fail("test_or - expected TypeError")
        except TypeError:
            pass
        try:
            a | Integers
            self.fail("test_or - expected TypeError")
        except TypeError:
            pass
        try:
            a | c
            self.fail("test_or - expected ValueError")
        except ValueError:
            pass

    def test_and(self):
        a=Set(initialize=[1,2,3])
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            Reals & Integers
            self.fail("test_and - expected TypeError")
        except TypeError:
            pass
        try:
            a & Integers
            self.fail("test_and - expected TypeError")
        except TypeError:
            pass
        try:
            a & c
            self.fail("test_and - expected ValueError")
        except ValueError:
            pass

    def test_xor(self):
        a=Set(initialize=[1,2,3])
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            Reals ^ Integers
            self.fail("test_xor - expected TypeError")
        except TypeError:
            pass
        try:
            a ^ Integers
            self.fail("test_xor - expected TypeError")
        except TypeError:
            pass
        try:
            a ^ c
            self.fail("test_xor - expected ValueError")
        except ValueError:
            pass

    def test_sub(self):
        a=Set(initialize=[1,2,3])
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            Reals - Integers
            self.fail("test_sub - expected TypeError")
        except TypeError:
            pass
        try:
            a - Integers
            self.fail("test_sub - expected TypeError")
        except TypeError:
            pass
        try:
            a - c
            self.fail("test_sub - expected ValueError")
        except ValueError:
            pass

    def test_mul(self):
        a=Set(initialize=[1,2,3])
        c=Set(initialize=[(1,2)])
        c.construct()
        try:
            Reals * Integers
            self.fail("test_mul - expected TypeError")
        except TypeError:
            pass
        try:
            a * Integers
            self.fail("test_mul - expected TypeError")
        except TypeError:
            pass
        try:
            a * 1
            self.fail("test_mul - expected TypeError")
        except TypeError:
            pass
        b = a * c

    def test_arrayset_construct(self):
        a=Set(initialize=[1,2,3])
        a.construct()
        b=Set(a, initialize=tmp_constructor)
        try:
            b.construct({4:None})
            self.fail("test_arrayset_construct - expected KeyError")
        except KeyError:
            pass
        b._constructed=False
        try:
            b.construct()
            self.fail("test_arrayset_construct - expected ValueError")
        except ValueError:
            pass
        b=Set(a,a, initialize=tmp_constructor)
        for i in b:
            self.assertEqual(i in a, True)
        try:
            b.construct()
            self.fail("test_arrayset_construct - expected ValueError")
        except ValueError:
            pass

    def test_prodset(self):
        a=Set(initialize=[1,2])
        a.construct()
        b=Set(initialize=[6,7])
        b.construct()
        c=a*b
        c.construct()
        self.assertEqual((6,2) in c, False)
        c=coopr.pyomo.base.sets._SetProduct(a,b)
        c.virtual=True
        self.assertEqual((6,2) in c, False)
        self.assertEqual((1,7) in c, True)
        #c=coopr.pyomo.base.sets._SetProduct()
        #c.virtual=True
        #c.construct()
        c=coopr.pyomo.base.sets._SetProduct(a,b,initialize={(1,7):None,(2,6):None})
        c.construct()
        c=coopr.pyomo.base.sets._SetProduct(a,b,initialize=(1,7))
        c.construct()

def virt_constructor(model, y):
    return RealSet(validate=lambda model, x: x>y)

class ArraySetVirtual(unittest.TestCase):

    def test_construct(self):
        a=Set(initialize=[1,2,3])
        a.construct()
        b=Set(a, initialize=virt_constructor)
        #b.construct()

if __name__ == "__main__":
    unittest.main()



# Imports
from coopr.pyomo import *

# Create the model object
model = AbstractModel()

# Sets
model.P = Set()

# Parameters
model.a = Param(model.P)
model.b = Param()
model.c = Param(model.P)
model.u = Param(model.P)

# Variables
model.X = Var(model.P)

# Objective
def Objective_rule(model):
    return sum([model.c[j]*model.X[j] for j in model.P])
model.Total_Profit = Objective(rule=Objective_rule, sense=maximize)

# Time Constraint
def Time_rule(model):
    return summation(model.X, denom=model.a) < model.b
model.Time = Constraint(rule=Time_rule)

# Limit Constraint
def Limit_rule(model, j):
    return (0, model.X[j], model.u[j])
model.Limit = Constraint(model.P, rule=Limit_rule)



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['IndexedComponent']

import component
from sets import Set


def process_setarg(arg):
    """
    Process argument and return an associated set object.
    """
    if isinstance(arg,Set):
        # Argument is a Set instance
        return arg
    elif isinstance(arg,IndexedComponent):
        raise TypeError, "Cannot index a component with a non-set component"
    else:
        try:
            # Argument has set_options attribute, which is used to initialize the set
            options = getattr(arg,'set_options')
            options['initialize'] = arg
            return Set(**options)
        except:
            pass
    # Argument is assumed to be an initialization function
    return Set(initialize=arg)


class IndexedComponent(component.Component):
    """
    This is the base class for all indexed modeling components.

    Constructor arguments:
        ctype       The class type for the derived subclass
        doc         A text string describing this component

    Private class attributes:
        _data       A dictionary from the index set to component data objects
        _index      The set of valid indices
        _index_set  A tuple of set objects that represents the index set
        _ndim       The dimension of the index set
    """

    def __init__(self, *args, **kwds):
        component.Component.__init__(self, **kwds)
        #
        self._ndim = 0
        self._index_set = None
        self._index = {}
        self._data = {}
        #
        if len(args) == 0:
            #
            # If no indexing sets are provided, generate a dummy index
            #
            self._index={None:None}
        elif len(args) == 1:
            #
            # If a single argument is provided, the define the index using that
            # argument
            #
            self._index = process_setarg(args[0])
        else:
            #
            # If multiple arguments are provided, define _index_set with a tuple
            # of set objects
            #
            tmp = []
            for arg in args:
                tmp.append(process_setarg(arg))
            self._index_set=tuple(tmp)
        #
        # Compute the dimension of the indexing sets
        #
        self._compute_dim()
        #
        # Setup indices for suffix data
        #
        self._suffix_declaration = {}
        self._suffix_value = {}

    def clear(self):
        """Clear the data in this component"""
        self._data = {}

    def __len__(self):
        return len(self._data)

    def __contains__(self, ndx):
        return ndx in self._data

    def keys(self):
        return self._data.keys()

    def values(self):
        return self._data.values()

    def __iter__(self):
        return self._data.__iter__()

    def iteritems(self):
        return self._data.iteritems()

    def itervalues(self):
        return self._data.itervalues()

    def __getitem__(self, ndx):
        """
        This method returns the data corresponding to the given index.
        """
        if ndx in self._data:
            return self._data[ndx]
        raise KeyError, "Unknown index in component '%s': %s" % ( self.name, str(ndx) )

    def index(self):
        """Return the index set"""
        return self._index

    def is_indexed(self):
        """Return true if this component is indexed"""
        return self._ndim > 0

    def dim(self):
        """Return the dimension of the index"""
        return self._ndim

    def _compute_dim(self):
        """Compute the dimension of the set"""
        if self._ndim is 0:
            if self._index_set is None:
                # A single index set
                if type(self._index) is dict:
                    # If the index set is dictionary, then the argument list was empty
                    self._ndim = 0
                else:
                    # If the index set is Pyomo Set, then use its dimen attribute
                    self._ndim = self._index.dimen
            else:
                # A multi-set index
                self._ndim = 0
                for iset in self._index_set:
                    # Each index set is a Pyomo Set, so use its dimen attribute
                    self._ndim += iset.dimen

    def declare_suffix(self, suffix, default=None, rule=None):
        """
        Declare a suffix attribute.
        """
        if suffix[0] == "_":
            msg = "Cannot define a suffix that begins with  '_'"
            raise AttributeError, msg
        if suffix in self._suffix_declaration:
            raise AttributeError, "Suffix %s is already defined" % suffix
        self._suffix_declaration[suffix] = (default,rule)
        #
        # If this variable has been constructed, then
        # generate this attribute for all variables
        #
        #if len(self._data) > 0:
            #self._suffix_value[suffix] = {}

    def suffix_defined(self, suffix):
        """
        Determine if a suffix has aready been declared.
        """
        return suffix in self._suffix_declaration

    def get_suffix_value(self, suffix, index=None):
        try:
            return self._suffix_value[suffix, index]
        except KeyError:
            default = self._suffix_declaration[suffix]
            if not default[0] is None:
                return default[0]
            if not default[1] is None:
                val = default[1](self)
                self._suffix_value[suffix, index] = val
                return val
            return None

    def set_suffix_value(self, suffix, value, index=None, define=False):
        if define and suffix not in self._suffix_declaration:
            self.declare_suffix(suffix)
        self._suffix_value[suffix, index] = value

